function [Vk, kOpt, Vy, kOptY] = Viterate(param, gridPoints, tol, IterMax)
% function [Vk, kOpt, Vy, kOptY] = Viterate(param, gridPoints, tol,
% IterMax)
%
% Calculates V
%
% param:      Parameters of the model
% gridPoints: Number of grid points to approximate V
% tol:        Numerical tolerance to stop iteration
% IterMax:    Maximal number of iterations


    % Initialization and parameter cheks
    % ------------------------------------

    if nargin <3
        tol = 0.000001;
    end

    if nargin < 2
        gridPoints = 1000;
    end

    if nargin <1
        error('Not enough input arguments');
    end



    if tol <= 0
        error('tol must be positive');
    end
    
    if nargin <4
        IterMax = 100;
    end
    

    
    IterMax = IterMax(1);

    param = checkParam(param);
    s = param.s;
    nOfStates = numel(s);
   
    
    % Calculate V0 with grid of y in the first column
    % -------------------------------------------------
    
     % with the Max is meant the maximum of the grid
       
    
    if numel(gridPoints)==1
        gridPoints = fix(gridPoints);

        if gridPoints <2
            error('More than 1 gridpoint is needed');
        end
        
         switch param.case
            case 'CobbDouglas'
                alpha = param.alpha;
                gamma = param.gamma;
                delta = param.delta;

                % makes range smaller but it needs to be checked
                % if optimal capital level goes above
                kMaxSub = (gamma/(min(param.beta)+delta))^(1/(1-alpha));

                % whole theoretically possible range
                kMax = (gamma*alpha/(1+delta) )^(1/(1-alpha)) ;
                                
                % weighted
                kMax = (1.1*kMax + 3*kMaxSub) / 4;
                fMax = f(kMax,0, param);
                
            case 'Polynom'
               
                kMax = 1;
                fMax = f(kMax,0, param);
              
             otherwise
                error('not existing case');
        end
       
        maxToCalc = 1.1*fMax;
        %maxToCalc = 1.01*max(kMax, fMax);

       scale = param.scale;
       scaledMax  = maxToCalc.^(1/scale);
        
       Vpre = zeros(gridPoints, nOfStates+1);
    
        Vpre(:,1) = (0:(scaledMax/(gridPoints-1)):scaledMax)';
        if scale~=1  
            Vpre(:,1) = Vpre(:,1).^scale;
        end
    else
          if numel(gridPoints)<10
              warning('Grid has less then 10 elements');
          end
          Vpre = zeros(numel(gridPoints), nOfStates+1);
          Vpre(:,1) = gridPoints;
    end
    
%     k = repmat(Vpre(:,1), 1, nOfStates);
%     state = 1:nOfStates;
%     state = repmat(state, numel(gridPoints), 1);

    % V in terms of y
    Vpre(:,2:end) = repmat(Vpre(:,1), 1, size(Vpre,2)-1);


    
    % Iteration over V_n
    % -------------------
    
    Vy    = Vpre;
    Vdiff = Inf;
    n=0;
    
    if param.plot
        styles = ['b-'; 'g-'; 'r-'; 'c-'; 'm-'; 'y-'; 'k-';'b.'; 'g.'; 'r.'; 'c.'; 'm.'; 'y.'; 'k.'];
        styles2 = ['k- '; 'k. '; 'k--'; 'k: ']; 
        plot(Vy(:,1), Vy(:,2), 'b.')
        hold all
    
        tic;
    end
    
    [index, weight, maxIndex]=calcIndexForVnY(Vpre(:,1), param);
    
    while n<IterMax & (Vdiff>tol | n<=1)
        %V(:,2:end) = Vn(k, state, Vpre, param);
        Vy = VnY(Vpre, param, index, weight, maxIndex);        
        
        Vdiff = max( vec(abs( Vy(:,2:end)- Vpre(:,2:end))));
        
        n = n+1;
        Vpre = Vy;
        
        if (Vdiff <= tol | n==(IterMax-1) )& nargout>1
            [Vy, ind] = VnY(Vpre, param, index, weight, maxIndex);        
        
            Vdiff = max( vec(abs( Vy(:,2:end)- Vpre(:,2:end))));
            n = n+1;
            Vpre = Vy;
        end
        

        
        if param.plot
            for ii = 2:size(Vy,2)
                subplot(2, size(Vy,2)-1, ii-1)
                hold all
                plot(Vy(:,1), Vy(:,ii), styles(mod(n, size(styles, 1))+1,1))
            end
            
            usedTime = toc /3600;
            sZwi = strcat('Iteration:  ', num2str(n),...
                '  V_n - V_(n-1):  ', num2str(Vdiff), ....
                '  used time: ',' ', ...
                num2str(usedTime,2), ' h');
             disp(sZwi);        
        end
    end
    
    
    % Calculate Vk
    % -------------
    % index may be higher than grid. Linearly interpolate above
    gridLength = size(Vpre, 1);
    if maxIndex > gridLength
        appendV = repmat(Vpre(end,:), maxIndex-gridLength,1);
        appendV = appendV + cumsum(repmat(Vpre(end,:)-Vpre(end-1,:), size(appendV, 1),1),1);
        Vpre = [Vpre;appendV];
    end

    % Calculate value function to each capital stock and state
    Vk = (1-weight).* Vpre(index) + weight .* Vpre(index+1);
    Vk = [Vy(:,1) Vk];
    
    % Produce output with kOpt
    %-------------------------
    
    if nargout>=2
        kOptY  = [Vy(:,1), Vy(ind)];
        if maxIndex > gridLength
            appendV = repmat(kOptY(end,:), maxIndex-gridLength,1);
            appendV = appendV + cumsum(repmat(kOptY(end,:)-kOptY(end-1,:), size(appendV, 1),1),1);
            temp = [kOptY;appendV];
        else
            temp = kOptY;
        end
        kOpt = (1-weight).* temp(index) + weight .* temp(index+1);
        kOpt = [Vy(:,1) kOpt];
    end
