function [V, Vbig] = VnKlausStyle(Vpre, param)
% function V = Vn(k, state, Vpre, param)
%
% maximizes V in an iterative way V=V_n and Vpre = V_n-1
%
% Vpre:      V from the last iteration
% param:     structure with the marameters of the model


    % Consistency Checks
    % ------------------
    
    param = checkParam(param);
    
    if nargin ~=2
        error('Wrong number of parameters');
    end
    
    % Initialization of Parameters
    % -----------------------------
    
    sMax = size(Vpre, 2)-1;
    gridPoints = size(Vpre, 1);
    
    k = Vpre(:,1);
    k = repmat(k, [1, sMax, gridPoints]);
    
    s = repmat(NaN, [gridPoints sMax gridPoints]);
    for ii = 1:sMax
       s(:,ii,:) = param.s(ii); 
    end    
    
    
    % Calculate V(n+1)
    % ----------------
    
    % calculate E(V(k_t+1, s_t+1))
    V = repmat(NaN, sMax, gridPoints);
    
    for ss = 1:sMax
        V(ss,:) = sum(Vpre(:, 2:end) .* ...
            repmat(param.p(:,ss)', gridPoints, 1), 2);
    end
    
    V = reshape(V, [1,sMax,gridPoints]);
    V = repmat(V, [gridPoints, 1, 1]);
    
    % calculate dividends
    d = reshape( Vpre(:,1), [1, 1, gridPoints]); % k(t+1)
    d = repmat(d, [gridPoints, sMax, 1]);
    d = f(k,  s, param) - d; % d = f-k(t+1)
    d(d<0) = -Inf;    
    V = d  + param.beta * V;
    
    if nargout >=2
        Vbig = V;
    end
    
    % Extract Maximum
    V = max(V, [], 3);
    V = [k(:,1,1), V];
    
    
    