function [Vy, ind] = VnY(VyPre, param, index, weight, maxIndex)
% function Vy = VnY(VyPre, param, index, weight, maxIndex)
%
% maximizes V in an iterative way V=V_n and Vpre = V_n-1
%
% Vpre:      V from the last iteration
% param:     structure with the marameters of the model


    % Consistency Checks
    % ------------------
    
    param = checkParam(param);
    
    if nargin <2 | nargin > 5 | nargin==3
        error('Wrong number of parameters');
    end

    
    % Initialize parameters
    % ----------------------
    
    p = param.p;
    nOfS = numel(param.s);
 
    gridLength = size(VyPre, 1);
    y = VyPre(:,1);
    
    % Calculate indices of of f(y, s) on y i.e. on which index we will end
    % up if we invest a capiatal k
    if nargin~=5
        [index, weight, maxIndex] = calcIndexForVnY(y, param);
    end
    
    
    % index may be higher than grid. Linearly interpolate above
    if maxIndex > gridLength
        appendV = repmat(VyPre(end,:), maxIndex-gridLength,1);
        appendV = appendV + cumsum(repmat(VyPre(end,:)-VyPre(end-1,:), size(appendV, 1),1),1);
        VyPre = [VyPre;appendV];
    end

    % Calculate value function to each capital stock and state
    Vk = (1-weight).* VyPre(index) + weight .* VyPre(index+1);
    Vk = [y Vk];
    
    % Calculate beta * E(V(k,s) | s-)
    pLarge = reshape(p, [1,nOfS,nOfS]);
    pLarge = repmat(pLarge, [size(Vk,1),1, 1]);
    EVk = repmat(Vk(:,2:end), [1,1, nOfS]);
    EVk = sum(pLarge .* EVk, 2);
    EVk = reshape(EVk, [size(Vk,1),nOfS,1]);
    
    beta = param.beta;
    if numel(beta)~=1
       if numel(beta)~= size(EVk, 2)
           error('Wrong size of beta');
       end
       beta = reshape(beta, [1 size(EVk, 2)]);
       beta = repmat(beta, [size(EVk,1), 1]);
    end
    EVk = [Vk(:,1), beta .* EVk;];
    
    
    % Find optimal k where V(y) is maximal by iterating over the whole grid
    % Return the highest V(y)
    % Return the index of the optimal capital if wanted.
    if nargout == 1
        if param.upper
            Vy =VyOptUpper(p, EVk);
        else
            Vy =VyOpt(p, EVk);
        end
    else
        if param.upper
            [Vy, ind] =VyOptUpper(p, EVk);
        else
            [Vy, ind] =VyOpt(p, EVk);
        end
    end

    Vy = [y, Vy];


   