/*
 * VyOptUpper.c - calculates next optimal V
 * Matlab usage:   [Vy, OptIndex] = VyOptUpper(p, beta*E(Vk))
 */

#include "mex.h"


/* Some definitions to make the life easier*/
#define P prhs[0]
#define EVK prhs[1]
#define VY plhs[0]
#define IND plhs[1]


void mexFunction( int nlhs,       mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
    double Vmax, Vnew;
    double *p, *vy, *evk, *ind;
    int iMax, i, j, actState, nStates, sizeGrid;
    
    /* Do some error checking */
    if (nrhs <2)
        mexErrMsgTxt("Missing input arguments.");
    else if (nrhs>3)
        mexErrMsgTxt("Too many input arguments.");
    
    if (nlhs>2)
        mexErrMsgTxt("Too many output arguments.");
    
    /* Get number of states */
    if(!mxIsNumeric(P) || mxIsComplex(P))
        mexErrMsgTxt("p has the wrong data type");
    else 
    {
        nStates = mxGetM(P);
        if (nStates!= mxGetN(P))
            mexErrMsgTxt("p has the wrong dimensions");
    }
    
     /* Get Grid Characteristics */
    if(!mxIsNumeric(EVK) || mxIsComplex(EVK) || mxGetN(EVK) != (nStates +1))
        mexErrMsgTxt("beta*EVk has the wrong data type or the wrong dimension");
    else 
    {
        sizeGrid = mxGetM(EVK);
    }
    
    /* Create empty Output */
    VY = mxCreateDoubleMatrix(sizeGrid, nStates, mxREAL);
    if (nlhs==2)
        IND = mxCreateDoubleMatrix(sizeGrid, nStates, mxREAL);
    
    /* Create Pointers */
    p = (double *) mxGetPr(P);
    evk = (double *) mxGetPr(EVK);    
    vy = (double *) mxGetPr(VY);
    if (nlhs==2)    
        ind = (double *) mxGetPr(IND);
    
    for (actState = 1; actState<=nStates; actState++)
    {
        for (j=0; j<sizeGrid; j++)
        {
            Vmax=-1;
            Vnew=-1;
            iMax=0;

            for (i=0; (i<sizeGrid) && (i<=j); i++)
            {
                Vnew = evk[j] - evk[i]+evk[actState*sizeGrid+i]; /* Total Output minus chosen captial = dividends*/
                
                if (Vnew>=Vmax)
                {
                    Vmax = Vnew;
                    iMax = i;
                }
            }

            vy[(actState-1)*sizeGrid+j] = Vmax;

            if (nlhs==2)
                ind[(actState-1)*sizeGrid+j] = iMax+1;
        }
    }
}
