function  [pDie, yGrowth, Vgrowth, Vvola, ExpRet, VolaRet]= ...
    calcVars(kOpt, V, param, maxHorizon, index)
% [pDie, yGrowth, Vgrowth, Vvola, ExpRet, VolaRet]= ...
%     calcVars(kOpt, V, param, maxHorizon, index)
% 
% Calculates probability of the dieing of a company within
% a certain time horizon

    % Initialization and Consistency checks
    % -------------------------------------

    param = checkParam(param);

    if nargin<4
        maxHorizon = 3;
    else
        maxHorizon = fix(maxHorizon);
    end

    if any(size(kOpt)~=size(V))
        error('kOpt and V need the same dimensions');
    end
    
    kOptFull =kOpt;
    VFull = V;
    
    if nargin== 5
        kOpt = kOpt(index,:);
        V    = V(index,:);
    end
    
    tol = 1e-10;
    


    % Determine all possible states
    % ------------------------------

    maxStates = size(kOpt,2)-1;
    states = NaN((maxStates)^(maxHorizon+1), maxHorizon+1);
    horizons = repmat(1:maxHorizon, size(states, 1), 1);

    for ii = 0:maxHorizon
        temp = vec(repmat((1:maxStates), size(states,1)/maxStates^(ii+1), 1));
        states(:,ii+1) = repmat(temp, maxStates^(ii), 1);
    end


    % Calculate Probability of scenarios
    % ----------------------------------
   
    temp = states(:,2:end);
    lagTemp = states(:, 1:(end-1));
    p = NaN(size(temp));
    for ss = 1:maxStates
        tempPos = lagTemp == ss;
        p(tempPos) = param.p(temp(tempPos),ss);
    end
    p = [ones(size(states, 1),1),p];
    p = cumprod(p, 2);

    temp    = repmat(maxStates, 1, maxHorizon+1);
    temp(1) = 1;
    temp = cumprod(temp, 2);
    temp = temp( (maxHorizon+1):-1:1);
    temp = repmat(temp, size(states,1), 1);

    p = p ./ temp;

    if any(abs(sum(p, 1)-maxStates)>0.00000001)
        error('p does not sum up to 1');
    end


    % Calculate the variables of interest
    % -----------------------------------

    pDie = NaN([size(kOpt,1), maxStates, maxHorizon]);
%     kAbs = NaN([size(kOpt,1), maxStates, maxHorizon+1]);
%     kRel = NaN([size(kOpt,1), maxStates, maxHorizon]);
%     kGrowth = NaN([size(kOpt,1), maxStates, maxHorizon]);
%     Vabs = NaN([size(kOpt,1), maxStates, maxHorizon+1]);
%     Vrel = NaN([size(kOpt,1), maxStates, maxHorizon]);
     Vgrowth = NaN([size(kOpt,1), maxStates, maxHorizon]);
%     VabsVola = NaN([size(kOpt,1), maxStates, maxHorizon+1]);
%     VrelVola = NaN([size(kOpt,1), maxStates, maxHorizon]);
    Vvola = NaN([size(kOpt,1), maxStates, maxHorizon]);
%     yRel = NaN([size(kOpt,1), maxStates, maxHorizon]);
    yGrowth = NaN([size(kOpt,1), maxStates, maxHorizon]);
    ExpRet  = NaN([size(kOpt,1), maxStates, maxHorizon]);
    VolaRet  = NaN([size(kOpt,1), maxStates, maxHorizon]);
    
    for kk = 1: size(kOpt, 1)
        kTemp = NaN(size(states));
        yTemp = NaN(size(states));
        Vtemp = NaN(size(states));
  
        
        kTemp(:,1)  = kOpt(kk, 1);
        yTemp(:,1)  = f(kTemp(:,1), param.s(states(:,1)), param);
        Vtemp(:,1)  = evalV(kTemp(:,1), states(:,1), VFull);
        warning('on', 'all');
        % Is a company bankrupt in period 0 -> give NaNs back
        bankrupt = tol >= f(repmat(kOpt(kk,1), size(param.s)), ...
           param.s, param);
    
    
        for t=2:(maxHorizon+1)
            kTemp(:,t) = calcKopt(kTemp(:,t-1), states(:,t-1), kOptFull);
            Vtemp(:,t) = evalV(kTemp(:,t), states(:,t), VFull);
            yTemp(:,t) = f(kTemp(:,t), param.s(states(:,t)), param);

        end
        
        
        temp = (Vtemp.^2<=tol) .* p;
        for s = 1:maxStates
            if bankrupt(s)
                pDie(kk, s,:) = NaN;
            else
                pDie(kk, s,:) = sum(temp(states(:,1)==s,2:end), 1);
            end                
        end    
        

        
%             temp = repmat(kTemp(:,1), 1, maxHorizon);
%             warning 'off'
%             temp = (kTemp(:,2:end)./temp).^(1./horizons)-1;
%             warning('on', 'all');
%             temp(isnan(temp) & kTemp(:,2:end)==0) = 0;
%             temp = temp.*p(:,2:end);
%             for s = 1:maxStates
%                 if bankrupt(s)
%                     kGrowth(kk, s,:) = NaN;
%                 else
%                     kGrowth(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
%                 end    
%             end
% 
%            
%            temp = repmat(kTemp(:,1), 1, maxHorizon);
%            warning 'off'
%            temp = (kTemp(:,2:end)./temp)-1;
%            warning('on', 'all');
%                                
%            temp(isnan(temp) & kTemp(:,2:end)==0) = 0;
%            temp = temp.*p(:,2:end);
%                 
%            for s = 1:maxStates
%                 if bankrupt(s)
%                     kRel(kk, s,:) = NaN;
%                 else
%                     kRel(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
%                 end    
%            end
%             
%            
%            temp = repmat(yTemp(:,1), 1, maxHorizon);
%            warning 'off'
%            temp = (yTemp(:,2:end)./temp)-1;
%            warning('on', 'all');
%                                
%            temp(isnan(temp) & yTemp(:,2:end)==0) = 0;
%            temp = temp.*p(:,2:end);
%                 
%            for s = 1:maxStates
%                 if bankrupt(s)
%                     yRel(kk, s,:) = NaN;
%                 else
%                     yRel(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
%                 end    
%            end
%             
%            
           temp = repmat(yTemp(:,1), 1, maxHorizon);
           hor = repmat(1:maxHorizon, size(temp, 1), 1);
           warning 'off'
           temp = (yTemp(:,2:end)./temp).^(1./hor)-1;
           warning('on', 'all');
                               
           temp(isnan(temp) & yTemp(:,2:end)==0) = 0;
           temp = temp.*p(:,2:end);
                
           for s = 1:maxStates
                if bankrupt(s)
                    yGrowth(kk, s,:) = NaN;
                else
                    yGrowth(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
                end    
            end
           

%             warning 'off'
%             temp = kTemp;
%             warning('on', 'all');
%             temp = temp.*p;
%             for s = 1:maxStates
%                 if bankrupt(s)
%                     kAbs(kk, s,:) = NaN;
%                 else
%                     kAbs(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
%                 end    
%             end
% 
            temp = repmat(Vtemp(:,1), 1, maxHorizon);
            warning 'off'
            temp = (Vtemp(:,2:end)./temp).^(1./horizons)-1;
            warning('on', 'all');
            temp(isnan(temp) & Vtemp(:,2:end)==0) = 0;
            temp = temp.*p(:,2:end);
            for s = 1:maxStates
                if bankrupt(s)
                    Vgrowth(kk, s,:) = NaN;
                else
                    Vgrowth(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
                end             
            end
% 
%             temp = repmat(Vtemp(:,1), 1, maxHorizon);
%             warning 'off'
%             temp = (Vtemp(:,2:end)./temp)-1;
%             warning('on', 'all');
%             temp(isnan(temp) & Vtemp(:,2:end)==0) = 0;
%             temp = temp.*p(:,2:end);
%             for s = 1:maxStates
%                 if bankrupt(s)
%                     Vrel(kk, s,:) = NaN;
%                 else
%                     Vrel(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
%                 end             
%             end
% 
% 
% 
%             warning 'off'
%             temp = Vtemp;
%             warning('on', 'all');
%             temp = temp.*p;
%             for s = 1:maxStates
%                 if bankrupt(s)
%                     Vabs(kk, s,:) = NaN;
%                 else
%                     Vabs(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
%                 end             
%             end
% 
% 
%             warning 'off'
%             temp = Vtemp;
%             warning('on', 'all');
%             temp = temp.^2.*p;
%             for s = 1:maxStates
%                 if bankrupt(s)
%                     VabsVola(kk, s,:) = NaN;
%                 else
%                     VabsVola(kk, s,:) = (vec(sum(temp(states(:,1)==s,:), 1)) - ...
%                         vec(Vabs(kk, s,:)).^2).^.5;
%                 end               
%             end
% 
%             
%             temp = repmat(Vtemp(:,1), 1, maxHorizon); 
%             warning 'off'
%             temp = (Vtemp(:,2:end)./temp)-1;
%             warning('on', 'all');
%             temp(isnan(temp) & Vtemp(:,2:end)==0) = 0;
%             temp = temp.^2.*p(:,2:end);
%             for s = 1:maxStates
%                 if bankrupt(s)
%                     VrelVola(kk, s,:) = NaN;
%                 else
%                     VrelVola(kk, s,:) = (vec(sum(temp(states(:,1)==s,:), 1)) - ...
%                         vec(Vgrowth(kk, s,:)).^2).^.5;
%                 end               
%             end
% 
            temp = repmat(Vtemp(:,1), 1, maxHorizon);
            warning 'off'
            temp = (Vtemp(:,2:end)./temp).^(1./horizons)-1;
            warning('on', 'all');
            temp(isnan(temp) & Vtemp(:,2:end)==0) = 0;
            temp = temp.^2.*p(:,2:end);
            for s = 1:maxStates
                if bankrupt(s)
                    Vvola(kk, s,:) = NaN;
                else
                    Vvola(kk, s,:) = sqrt(...
                        vec(sum(temp(states(:,1)==s,:), 1)) - ...
                        vec(Vgrowth(kk, s,:)).^2);
                end               
            end
        
            
            RItemp = Vtemp(:,1)-(yTemp(:,1)-kTemp(:,1));
            RItemp = repmat(RItemp, 1, size(Vtemp, 2)-1);
            RItemp = Vtemp(:,2:end)./RItemp;
            RItemp = RItemp .^ (1./repmat(1:maxHorizon, size(RItemp, 1), 1));
            
            for s = 1:maxStates
                if bankrupt(s)
                    ExpRet(kk, s,:) = NaN;
                    VolaRet(kk, s,:) = NaN;
                else
                    ExpRet(kk, s,:) = sum(p(states(:,1)==s,2:end) .* ...
                        RItemp(states(:,1)==s,:));
                    VolaRet(kk, s,:) =   sqrt(...
                        sum(p(states(:,1)==s,2:end) .* RItemp(states(:,1)==s,:).^2) -...
                        squeeze(ExpRet(kk,s,:))'.^2);
                end               
            end
            

        
    end
    

    
 




    




