function  [pDie, yGrowth, Vgrowth, Vvola, ExpRet, VolaRet, ExpROI, VolaROI]= ...
    calcVarsY(kOptY, Vy, param, maxHorizon, index)
% [pDie, yGrowth, Vgrowth, Vvola, ExpRet, VolaRet]= ...
%    calcVarsY(kOpt, V, param, maxHorizon, index)
% 
% Calculates probability of the dieing of a company within
% a certain time horizon

    % Initialization and Consistency checks
    % -------------------------------------

    param = checkParam(param);

    if nargin<4
        maxHorizon = 3;
    else
        maxHorizon = fix(maxHorizon);
    end

    if any(size(kOptY)~=size(Vy))
        error('kOpt and V need the same dimensions');
    end
    
    kOptFull =kOptY;
    VFull = Vy;
    
    if nargin== 5
        kOptY = kOptY(index,:);
        Vy    = Vy(index,:);
    end
    
    tol = 1e-10;
    


    % Determine all possible states
    % ------------------------------

    maxStates = size(kOptY,2)-1;
    states = NaN((maxStates)^(maxHorizon+1), maxHorizon+1);
    horizons = repmat(1:maxHorizon, size(states, 1), 1);

    for ii = 0:maxHorizon
        temp = vec(repmat((1:maxStates), size(states,1)/maxStates^(ii+1), 1));
        states(:,ii+1) = repmat(temp, maxStates^(ii), 1);
    end


    % Calculate Probability of scenarios
    % ----------------------------------
   
    temp = states(:,2:end);
    lagTemp = states(:, 1:(end-1));
    p = NaN(size(temp));
    for ss = 1:maxStates
        tempPos = lagTemp == ss;
        p(tempPos) = param.p(temp(tempPos),ss);
    end
    p = [ones(size(states, 1),1),p];
    p = cumprod(p, 2);

    temp    = repmat(maxStates, 1, maxHorizon+1);
    temp(1) = 1;
    temp = cumprod(temp, 2);
    temp = temp( (maxHorizon+1):-1:1);
    temp = repmat(temp, size(states,1), 1);

    p = p ./ temp;

    if any(abs(sum(p, 1)-maxStates)>0.00000001)
        error('p does not sum up to 1');
    end


    % Calculate the variables of interest
    % -----------------------------------

    pDie = NaN([size(kOptY,1), maxStates, maxHorizon]);
    Vgrowth = NaN([size(kOptY,1), maxStates, maxHorizon]);
    Vvola = NaN([size(kOptY,1), maxStates, maxHorizon]);
    yGrowth = NaN([size(kOptY,1), maxStates, maxHorizon]);
    ExpRet  = NaN([size(kOptY,1), maxStates, maxHorizon]);
    VolaRet  = NaN([size(kOptY,1), maxStates, maxHorizon]);
    ExpROI  = NaN([size(kOptY,1), maxStates, maxHorizon]);
    VolaROI  = NaN([size(kOptY,1), maxStates, maxHorizon]);
    
    for kk = 1: size(kOptY, 1)
        kTemp = NaN(size(states));
        yTemp = NaN(size(states));
        Vtemp = NaN(size(states));
  
        % Calculate k, y and V in the initial period
        yTemp(:,1)  = kOptY(kk, 1);
        kTemp(:,1)  = calcKopt(yTemp(:,1), states(:,1), kOptFull);
        Vtemp(:,1)  = evalV(yTemp(:,1), states(:,1), VFull);
        warning('on', 'all');
    
        % Is a company bankrupt in period 0 -> give NaNs back
        bankrupt = tol >= repmat(kOptY(kk,1), size(param.s));
        
        
        % Calculate k, y and V in the future periods
        for t=2:(maxHorizon+1)
            yTemp(:,t) = f(kTemp(:,t-1), param.s(states(:,t)), param);
            kTemp(:,t) = calcKopt(yTemp(:,t), states(:,t), kOptFull);
            Vtemp(:,t) = evalV(yTemp(:,t), states(:,t), VFull);          
        end
        
        % pDie
        temp = (Vtemp.^2<=tol) .* p;
        for s = 1:maxStates
            if bankrupt(s)
                pDie(kk, s,:) = NaN;
            else
                pDie(kk, s,:) = sum(temp(states(:,1)==s, 2:end), 1);
            end                
        end    
        
        
           % Expected Growth Y       
           temp = repmat(yTemp(:,1), 1, maxHorizon);
           hor = repmat(1:maxHorizon, size(temp, 1), 1);
           warning 'off'
           temp = (yTemp(:,2:end)./temp).^(1./hor)-1;
           warning('on', 'all');
                               
           temp(isnan(temp) & yTemp(:,2:end)==0) = 0;
           temp = temp.*p(:,2:end);
                
           for s = 1:maxStates
                if bankrupt(s)
                    yGrowth(kk, s,:) = NaN;
                else
                    yGrowth(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
                end    
           end
           
           

            % Growth and Volatility of V
            temp = repmat(Vtemp(:,1), 1, maxHorizon);
            warning 'off'
            temp = (Vtemp(:,2:end)./temp).^(1./horizons)-1;
            warning('on', 'all');
            temp(isnan(temp) & Vtemp(:,2:end)==0) = 0;
            temp = temp.^2.*p(:,2:end);
            for s = 1:maxStates
                if bankrupt(s)
                    Vgrowth(kk, s,:) = NaN;
                    Vvola(kk, s,:) = NaN;
                else
                    Vgrowth(kk, s,:) = sum(temp(states(:,1)==s,:), 1);
                    Vvola(kk, s,:) = sqrt(...
                        vec(sum(temp(states(:,1)==s,:), 1)) - ...
                        vec(Vgrowth(kk, s,:)).^2);
                end               
            end
        
            
            RItemp = Vtemp(:,1)-(yTemp(:,1)-kTemp(:,1));
            RItemp = repmat(RItemp, 1, size(Vtemp, 2)-1);
            RItemp = Vtemp(:,2:end)./RItemp;
            RItemp = RItemp .^ (1./repmat(1:maxHorizon, size(RItemp, 1), 1));
            
            for s = 1:maxStates
                if bankrupt(s)
                    ExpRet(kk, s,:) = NaN;
                    VolaRet(kk, s,:) = NaN;
                else
                    ExpRet(kk, s,:) = sum(p(states(:,1)==s,2:end) .* ...
                        RItemp(states(:,1)==s,:));
                    VolaRet(kk, s,:) =   sqrt(...
                        sum(p(states(:,1)==s,2:end) .* RItemp(states(:,1)==s,:).^2) -...
                        squeeze(ExpRet(kk,s,:))'.^2);
                end               
            end
            
            ROItemp = kTemp(:,1);
            ROItemp = repmat(ROItemp, 1, size(Vtemp, 2)-1);
            ROItemp = Vtemp(:,2:end)./ROItemp;
            ROItemp = ROItemp .^ (1./repmat(1:maxHorizon, size(ROItemp, 1), 1));
            
            for s = 1:maxStates
                if bankrupt(s)
                    ExpROI(kk, s,:) = NaN;
                    VolaROI(kk, s,:) = NaN;
                else
                    ExpROI(kk, s,:) = sum(p(states(:,1)==s,2:end) .* ...
                        ROItemp(states(:,1)==s,:));
                    VolaROI(kk, s,:) =   sqrt(...
                        sum(p(states(:,1)==s,2:end) .* ROItemp(states(:,1)==s,:).^2) -...
                        squeeze(ExpROI(kk,s,:))'.^2);
                end               
            end
            

        
    end
    
