function param = checkParam(param)
% function param = checkParameter(param)
%
% checks the parameter, if they are reasonable and have the right size

   
    beta = param.beta;
   
    p = param.p;
    param.s = vec(param.s);
    s = param.s;
    
    try
        scale = param.scale;
    catch
        param.scale = 1;
        scale=1;
    end
    
    try
        upper = param.upper;
    catch
        param.upper = false;
        upper = false;
    end
    
        
    try
        checked = param.checked;
    catch
        param.checked = false;
        checked = false;
    end
       
    
    % Some Consitency checks
    % ----------------------
    
    if any(abs(sum(p, 1)-1)>0.0000001)
        error('probabilities have to add up to 1');
    end
    

    
    if any(size(p)~=numel(s))
        error('wrong dimension of s or p');
    end
          
    if any(any(p<0))
        error('probabilities have to be positive');
    end
    
        
    if any(beta <=0 | beta>=1)  | (numel(beta) ~= 1 & ...
            any(size(beta) ~= [numel(s), 1]))
        error('beta has to be between 0 and 1');
    end
    
    if any(scale <=0) | numel(scale) ~= 1
        error('scale has to be a positive scalar');
    end
    
    if any(~islogical(checked)) | numel(checked) ~= 1        
        error('checked has to be a scalar logical value');
    end
    
    if any(~islogical(upper)) | numel(upper) ~= 1        
        error('upper has to be a scalar logical value');
    end
    
    if ~isfield(param, 'plot')
        param.plot = true;
    end
    
    if ~islogical(param.plot)
        param.plot = true;
    end
    
    switch param.case
        
        case 'CobbDouglas'
            alpha = param.alpha;
            gamma = param.gamma;
            delta = param.delta;
            if any(alpha <=0 | alpha >= 1) | numel(alpha) ~= 1
                error('alpha has to be between 0 and 1');
            end
    
            if any(delta <=0 | delta >= 1) | numel(delta) ~= 1
                error('delta has to be between 0 and 1');
            end
    
            if any(gamma <=0)  | numel(gamma) ~= 1
                error('gamma has to be positive');
            end
            
        case 'Polynom'
            
            param.coef = param.coef;
            if numel(param.coef)<0
                error('coef has to exist in the polynomial case');
            end
            
        otherwise
            error('The chosen case does not exist')
    end
    
    if any(s<0)
        warning('s should be non-negative');
    end
    
    param.checked = true;