function [pDie1, yGrowth1, Vgrowth1, Vvola1, ExpRet1, VolaRet1]=plotLC(kOpt, V, param, index, folder)
% function plotLC(kOpt, V, param, index, folder)
%
% plots graphs to the Life Cycle of companies

param = checkParam(param);
maxStates = size(kOpt,2)-1;

lw = 2;

if nargin<3
    error('not enough input arguments');
end

if nargin>=4
    kOpt = kOpt(index,:);
    V    = V(index,:);
end

if nargin<5
    folder = 'lastRun';
end

% if 7== exist('f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\','dir')
%     defaultPath = 'f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\';
% elseif 7== exist('C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\','dir')
%     defaultPath = 'C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\';
% else
    mkdir(cd, 'Graphs');
    defaultPath = strcat(cd, '\Graphs\');
% end

path = strcat(defaultPath, folder, '\');


lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes)
xLimits = [min(V(index,1)) max(V(index,1))]

% % relative Weahlth with confidence Intervals
% figure;
% relWealth = m.wealth ./ repmat(sum(m.wealth, 2), 1, numOfInvestors);
% aMean = mean(relWealth, 3);
% aLowQ = quantileDim(relWealth, alpha/2, 3);
% aHighQ = quantileDim(relWealth, 1-alpha/2, 3);
% plot(index, aMean(index,:,:), index, aLowQ(index,:,:), ':', ...
%     index, aHighQ(index,:,:), ':');
% axis([min(index), max(index), 0, 1]);
% title('Relative Wealth');
% xlabel(cat(2, 'Mean and  ', num2str(100*(1-alpha)), '% Confidence Interval'));
% legends(m.struct.strategies(m.struct.investors,:));


% Overview over functions
% ------------------------

figure;
 
subplot(2,2,1);
plot(V(:,1), V(:,2:end), 'LineWidth', lw)
title('(a) Value Function');
ylabel('V');
xlabel('k');
xlim(xLimits);
 
subplot(2,2,3);
plot(kOpt(:,1), kOpt(:,2:end), 'LineWidth', lw)
title('(c) Optimal Capital in the Next Period');
ylabel('k_{t+1}^*');
xlabel('k');
xlim(xLimits);

[dStar, fStar] = calcDiv(kOpt, param);

subplot(2,2,2);
plot(dStar(:,1), dStar(:,2:end), 'LineWidth', lw)
title('(b) Optimal Dividend Payment');
ylabel('d^*');
xlabel('k');
xlim(xLimits);
 
subplot(2,2,4);
plot(fStar(:,1), fStar(:,2:end), 'LineWidth', lw)
title('(d) Production Function');
ylabel('f(k, s)');
xlabel('k');
xlim(xLimits);

try
    saveas(gcf, strcat(path,'LcOverview') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'LcOverview') , 'epsc');
end


% Payout Quota
% ------------

figure;
warning 'off'  
plot(kOpt(2:end,1), dStar(2:end,2:end)./fStar(2:end,2:end),...
    'LineWidth', lw);
warning('on', 'all');
ylabel('d^*/f(k, s)');
xlabel('k');
title('Payout Quota');
xlim(xLimits);

try
    saveas(gcf, strcat(path,'LcPayoutQuota') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'LcPayoutQuota') , 'epsc');
end



T = 5;
%showPeriods=[1 T];
N = size(kOpt,1);
showPeriods = [1 3 T];
reduction=1;

reduction = max(1, floor(N/6000));
reducedIndex = 1:reduction:N;
kOptReduced = kOpt(reducedIndex,:);
VReduced = V(reducedIndex,:);

[pDie, yGrowth, Vgrowth, Vvola, ExpRet, VolaRet]=  calcVars(kOptReduced, VReduced, param, T);

for s=1:maxStates
    
    
    
%     % Probability to Die
%     % ------------------
%     figure;
%     title(strcat('State ', num2str(s)));
%     subplot(2,2,1);
%     plot(kOptReduced(:,1), squeeze(pDie(:,s,showPeriods)), 'LineWidth', lw)
%     title('(a) Probability to Die');
%     xlabel('k');
%     xlim(xLimits);
%     
% 
%     % Growth of Capital
%     % -----------------
%     subplot(2,2,2);
%     plot(kOptReduced(:,1), squeeze(kGrowth(:,s,showPeriods)), 'LineWidth', lw)
%     title('(c) Exp. Annualized Growth Rate of Capital');
%     xlabel('k');
%     
%     
%     % Growth Rate of Value
%     % --------------------
%     subplot(2,2,3);
%     plot(kOptReduced(:,1), squeeze(Vgrowth(:,s,showPeriods)), 'LineWidth', lw)
%     title('(b) Exp. Annualized Growth Rate of V');
%     xlabel('k');
%     xlim(xLimits);
%     
% 
%     % Volatility in Value
%     % --------------------
%     subplot(2,2,4);
%     plot(kOptReduced(:,1), squeeze(Vvola(:,s,showPeriods)), 'LineWidth', lw)
%     title('(d) Volatility of Exp. Annualized Growth Rate of V');
%     xlabel('k');
%     xlim(xLimits);
%     
%     
%     % Save Graph
%     % ----------
%     
%     filename = 'LcGrowthEtcState';
%     filename = strcat(filename, num2str(s));
%     try
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     catch %if there is no directory create one
%         mkdir(defaultPath, folder)
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     end
    
    
    % Everything in absolute Values
    % ------------------------------
    
%     figure;
%     title(strcat('State ', num2str(s)));
%     
%     % Probability to Die
%     % ------------------
%     subplot(2,2,1);
%     plot(kOptReduced(:,1), squeeze(pDie(:,s,showPeriods)), 'LineWidth', lw)
%     title('(a) Probability to Die');
%     xlabel('k');
%     xlim(xLimits);
%     
% 
%     % Capital
%     % --------
%     subplot(2,2,2);
%     plot(kOptReduced(:,1), squeeze(kAbs(:,s,1+showPeriods)), 'LineWidth', lw)
%     title('(c) Exp. Capital');
%     xlabel('k');
%     xlim(xLimits);
%     
%     
%     % Value
%     % -----
%     subplot(2,2,3);
%     plot(kOptReduced(:,1), squeeze(Vabs(:,s,1+showPeriods)), 'LineWidth', lw)
%     title('(b) Exp. NPV');
%     xlabel('k');
%     
% 
%     % Vola of Value
%     % --------------
%     subplot(2,2,4);
%     plot(kOptReduced(:,1), squeeze(VabsVola(:,s,1+showPeriods)), 'LineWidth', lw)
%     title('(d) Volatility of NPV');
%     xlabel('k');
%     xlim(xLimits);
%     
%     
%     % Save Graph
%     % ----------
%     
%     filename = 'LcAbsEtcState';
%     filename = strcat(filename, num2str(s));
%     try
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     catch %if there is no directory create one
%         mkdir(defaultPath, folder)
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     end
%     
    
%     % Everything in relative Values
%     % ------------------------------
%     
%     figure;
%     title(strcat('State ', num2str(s)));
%     
%     % Probability to Die
%     % ------------------
%     subplot(2,2,1);
%     plot(kOptReduced(:,1), squeeze(pDie(:,s,showPeriods)), 'LineWidth', lw)
%     title('(a) Probability to Die');
%     xlabel('k');
%     xlim(xLimits);
%     
%     % Exp. relative Capital
%     % ----------------------
%     subplot(2,2,2);
%     plot(kOptReduced(:,1), squeeze(kRel(:,s,showPeriods)), 'LineWidth', lw)
%     
%     temp = max(max(squeeze(kRel(:,s,showPeriods))))
%     temp = min(temp, 3)
%     ylim([-1, temp])
%     title('(c) Exp. Relative Change of Capital');
%     xlabel('k');
%     
%     
%     % Exp. relative Value
%     % --------------------
%     subplot(2,2,3);
%     plot(kOptReduced(:,1), squeeze(Vrel(:,s,showPeriods)), 'LineWidth', lw)
%     title('(b) Exp. Relative Change of NPV');
%     xlabel('k');
%     xlim(xLimits);
%     
% 
%     % Volatility in relative change of Value
%     % ------------------------------------------
%     subplot(2,2,4);
%     plot(kOptReduced(:,1), squeeze(VrelVola(:,s,showPeriods)), 'LineWidth', lw)
%     title('(d) Volatility of Relative Change in NPV');
%     xlabel('k');
%     xlim(xLimits);
%     
%     
%     
%     % Save Graph
%     % ----------
%     
%     filename = 'LcRelEtcState';
%     filename = strcat(filename, num2str(s));
%     try
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     catch %if there is no directory create one
%         mkdir(defaultPath, folder)
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     end
%     
%     % Everything in relative Values and in y
%     % ---------------------------------------
%     
%     figure;
%     title(strcat('State ', num2str(s)));
%     y = f(kOptReduced(:,1), param.s(s), param);
%     xyLimits = [min(y) max(y)];
%     if (any(isnan(xyLimits))| xyLimits(2)-xyLimits(1)<=0)
%         xyLimits =xLimits
%     end
%     
%     
%     % Probability to Die
%     % ------------------
%     subplot(2,2,1);
%     plot(y, squeeze(pDie(:,s,showPeriods)), 'LineWidth', lw)
%     title('(a) Probability to Die');
%     xlabel('y');
%     %ylim([-0.0001, max(quantile(squeeze(pDie(:,s,showPeriods)),.95))]);
%     ylim([-0.0001, 1])
%     xlim(xyLimits);
%     
%     % Exp. relative Endowment
%     % -------------------------
%     subplot(2,2,2);
%     plot(y, squeeze(yRel(:,s,showPeriods)), 'LineWidth', lw)
%     temp = max(max(squeeze(yRel(:,s,showPeriods))));
%     temp = min(temp, 3);
%     ylim([0, 1.5])
%     title('(c) Exp. Relative Change of Endowment');
%     xlabel('y');
%     xlim(xyLimits);
%     
%     
%     % Exp. relative Value
%     % --------------------
%     subplot(2,2,3);
%     plot(y, squeeze(Vrel(:,s,showPeriods)), 'LineWidth', lw)
%     title('(b) Exp. Relative Change of NPV');
%     xlabel('y');
%     ylim([-0.5, 1.5])
%     xlim(xyLimits);
% 
%     % Volatility in relative change of Value
%     % ------------------------------------------
%     subplot(2,2,4);
%     plot(y, squeeze(VrelVola(:,s,showPeriods)), 'LineWidth', lw)
%     title('(d) Volatility of Relative Change in NPV');
%     xlabel('y');
%     ylim([-0.05, 2])
%     xlim(xyLimits);
%     
%     % Save Graph
%     % ----------
%     
%     filename = 'LcRelYState';
%     filename = strcat(filename, num2str(s));
%     try
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     catch %if there is no directory create one
%         mkdir(defaultPath, folder)
%         saveas(gcf, strcat(path,filename) , 'epsc');
%     end

    
    % Everything in growth and in y
    % ---------------------------------------
    
    figure;
    title(strcat('State ', num2str(s)));
    y = f(kOptReduced(:,1), param.s(s), param);
    xyLimits = [min(y) max(y)];
    if (any(isnan(xyLimits))| xyLimits(2)-xyLimits(1)<=0)
        xyLimits =xLimits
    end
    
    
    % Probability to Die
    % ------------------
    subplot(2,2,1);
    plot(y, squeeze(pDie(:,s,showPeriods)), 'LineWidth', lw)
    title('(a) Probability to Die');
    xlabel('y');
    ylim([-0.0001 1.1*max(quantile(squeeze(pDie(:,s,showPeriods)),.95))]);
    xlim(xyLimits);
    
    
    
    % Exp. Growth of  Endowment
    % -------------------------
    subplot(2,2,2);
    plot(y, squeeze(yGrowth(:,s,showPeriods)), 'LineWidth', lw)
    temp = max(max(squeeze(yGrowth(:,s,showPeriods))));
    temp = min(1.05*temp, 2);
    ylim([0, temp]);
    title('(b) Exp. Growth of Endowment');
    xlabel('y');
    xlim(xyLimits);
    
    
    
    % Exp. Growth of Value
    % --------------------
    subplot(2,2,3);
    plot(y, squeeze(Vgrowth(:,s,showPeriods)), 'LineWidth', lw)
    title('(c) Exp. Growth of NPV');
    xlabel('y');
    temp = max(quantile(squeeze(Vgrowth(:,s,showPeriods)), .95));
    temp = min(1.05*temp, 2);
    ylim([0, temp]);
    xlim(xyLimits);

    
    
    % Volatility in Growth of Value
    % ------------------------------------------
    subplot(2,2,4);
    plot(y, squeeze(Vvola(:,s,showPeriods)), 'LineWidth', lw)
    title('(d) Volatility of Growth in NPV');
    xlabel('y');
    ylim([-0.005, 0.8])
    xlim(xyLimits);
    
    
    
    % Save Graph
    % ----------
    
    filename = 'LcGrowthYState';
    filename = strcat(filename, num2str(s));
    try
        saveas(gcf, strcat(path,filename) , 'epsc');
    catch %if there is no directory create one
        mkdir(defaultPath, folder)
        saveas(gcf, strcat(path,filename) , 'epsc');
    end
    
    
    
    
    
    % Mean Variance Diagram
    % ----------------------
    
    figure;
    temp = 5;
    plot(squeeze(ExpRet(:,s,temp)),squeeze(VolaRet(:,s,temp)), 'LineWidth', lw);
    
    filename = 'MeanVarianceT1';
    filename = strcat(filename, num2str(s));
    try
        saveas(gcf, strcat(path,filename) , 'epsc');
    catch %if there is no directory create one
        mkdir(defaultPath, folder)
        saveas(gcf, strcat(path,filename) , 'epsc');
    end
    
    
    
    if s==1
        pDie1 = [y squeeze(pDie(:,1,:))];
        yGrowth1 = [y squeeze(yGrowth(:,1,:))];
        Vgrowth1 = [y squeeze(Vgrowth(:,1,:))];
        Vvola1 = [y squeeze(Vvola(:,1,:))];
        ExpRet1 = [y squeeze(ExpRet(:,1,:))];
        VolaRet1 = [y squeeze(VolaRet(:,1,:))];
    end
end




function [numOfX, numOfY] = positionsInSubPlot(numOfGraph)
    % determines position in a subplot out of the number of graphs
    
    numOfGraph = ceil(numOfGraph);
    if numOfGraph<1
        error('numOfGraph must be at least 1')
    elseif numOfGraph == 1
        numOfX = 1; numOfY = 1;
    elseif numOfGraph == 2
        numOfX = 2; numOfY = 1;
    elseif numOfGraph == 3
        numOfX = 3; numOfY = 1;
    elseif numOfGraph == 4
        numOfX = 2; numOfY = 2;
    elseif numOfGraph == 5 | numOfGraph == 6
        numOfX = 3; numOfY = 2;
    elseif numOfGraph == 8 | numOfGraph == 7
        numOfX = 4; numOfY = 2;
    elseif numOfGraph == 9 
        numOfX = 3; numOfY = 3;
    elseif numOfGraph == 10
        numOfX = 5; numOfY = 2;
    elseif numOfGraph == 11 | numOfGraph == 12
        numOfX = 4; numOfY = 3;
    elseif numOfGraph == 13 | numOfGraph == 14 | numOfGraph == 15
        numOfX = 5; numOfY = 3;
    elseif numOfGraph == 16 
        numOfX = 4; numOfY = 4;
    elseif numOfGraph == 17 | numOfGraph == 18 
        numOfX = 6; numOfY = 3;
    elseif numOfGraph == 19 | numOfGraph == 20 
        numOfX = 5; numOfY = 4;
    elseif numOfGraph == 21 
        numOfX = 7; numOfY = 3;
    elseif numOfGraph == 22 | numOfGraph == 23 | numOfGraph == 24
        numOfX = 6; numOfY = 4;
    elseif numOfGraph == 25
        numOfX = 5; numOfY = 5;
    elseif numOfGraph == 26 | numOfGraph == 27 | numOfGraph == 28
        numOfX = 7; numOfY = 4;
    elseif numOfGraph == 29 | numOfGraph == 30 
        numOfX = 6; numOfY = 5;
    else
        error('To many graphs (more than 30)');
    end
    
function legends(agentNames, location)
    if nargin==1
        location = 0;
    end

    if size(agentNames, 1) == 1
        legend(agentNames(1,:), location);
    elseif size(agentNames, 1) == 2
        legend(agentNames(1,:), agentNames(2,:), location);
    elseif size(agentNames, 1) == 3
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), location);
    elseif size(agentNames, 1) == 4
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), location);
    elseif size(agentNames, 1) == 5
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), location);
     elseif size(agentNames, 1) == 6
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
             agentNames(6,:), location);
     elseif size(agentNames, 1) == 7
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
            agentNames(6,:), agentNames(7,:), location);
    else
        error('to many legends');
    end
    