function [pDie1, yGrowth1, Vgrowth1, Vvola1, ExpRet1, VolaRet1]=plotLCKSingle(kOpt, V, param, index, folder)
% function plotLC(kOpt, V, param, index, folder)
%
% plots graphs to the Life Cycle of companies

param = checkParam(param);
maxStates = size(kOpt,2)-1;

lw = 2;

if nargin<3
    error('not enough input arguments');
end

if nargin>=4
    kOpt = kOpt(index,:);
    V    = V(index,:);
end

if nargin<5
    folder = 'lastRun';
end

% if 7== exist('f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\','dir')
%     defaultPath = 'f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\';
% elseif 7== exist('C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\','dir')
%     defaultPath = 'C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\';
% else
    mkdir(cd, 'Graphs');
    defaultPath = strcat(cd, '\Graphs\');
% end


path = strcat(defaultPath, folder, '\');

filename=strcat(path,'data.xlsx');

if (size(kOpt, 2)==3)
    colnames = {'k', 'S=1', 'S=2'};
elseif (size(kOpt, 2)==4)
    colnames = {'k', 'S=1', 'S=2', 'S=3'};
elseif (size(kOpt, 2)==5)
    colnames = {'k', 'S=1', 'S=2', 'S=3', 'S=4'};
else
    colnames = {'k', 'S=1', 'S=2', 'S=3', 'S=4', 'S=5', '...'};
end

lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes);
xLimits = [min(V(index,1)) max(V(index,1))];



% Overview over functions
% ------------------------

figure;
plot(V(:,1), V(:,2:end), 'LineWidth', lw)
% title('(a) Value Function');
% ylabel('V');
% xlabel('k');
xlim(xLimits);

saveToTXT(strcat(path,'LcVk', '.txt'), V, 'kV');
saveToXLSX(filename,'LcVk', V, colnames, {'Value function depending on the capital'});
try
    saveas(gcf, strcat(path,'LcVk') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'LcVk') , 'epsc');
end
 
plot(kOpt(:,1), kOpt(:,2:end), 'LineWidth', lw)
% title('(c) Optimal Capital in the Next Period');
% ylabel('k_{t+1}^*');
% xlabel('k');
xlim(xLimits);

saveToTXT(strcat(path,'LcKk', '.txt'), kOpt, 'k K');
saveToXLSX(filename,'LcKk', kOpt, colnames, {'Optimal capital level given a capital level in the previous period'});
try
    saveas(gcf, strcat(path,'LcKk') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'LcKk') , 'epsc');
end

[dStar, fStar] = calcDiv(kOpt, param);


plot(dStar(:,1), dStar(:,2:end), 'LineWidth', lw)
% title('(b) Optimal Dividend Payment');
% ylabel('d^*');
% xlabel('k');
xlim(xLimits);

saveToTXT(strcat(path,'LcDk', '.txt'), dStar, 'kd');
saveToXLSX(filename,'LcDk', dStar, colnames, {'Optimal dividends given a capital level in the previous period'});
try
    saveas(gcf, strcat(path,'LcDk') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'LcDk') , 'epsc');
end
 
% subplot(2,2,4);
plot(fStar(:,1), fStar(:,2:end), 'LineWidth', lw)
% title('(d) Production Function');
% ylabel('f(k, s)');
% xlabel('k');
xlim(xLimits);

saveToTXT(strcat(path,'LcFk', '.txt'), fStar, 'kf');
saveToXLSX(filename,'LcFk', fStar, colnames, {'Production function given a capital level'});
try
    saveas(gcf, strcat(path,'LcFk') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'LcFk') , 'epsc');
end


% Payout Quota
% ------------

figure;
warning 'off'  
plot(kOpt(2:end,1), dStar(2:end,2:end)./fStar(2:end,2:end),...
    'LineWidth', lw);
warning('on', 'all');
ylabel('d^*/f(k, s)');
xlabel('k');
title('Payout Quota');
xlim(xLimits);
saveToTXT(strcat(path,'LcPayoutQuota', '.txt'), [kOpt(2:end,1), dStar(2:end,2:end)./fStar(2:end,2:end)], 'kD');
try
    saveas(gcf, strcat(path,'LcPayoutQuota') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'LcPayoutQuota') , 'epsc');
end



end




function [numOfX, numOfY] = positionsInSubPlot(numOfGraph)
    % determines position in a subplot out of the number of graphs
    
    numOfGraph = ceil(numOfGraph);
    if numOfGraph<1
        error('numOfGraph must be at least 1')
    elseif numOfGraph == 1
        numOfX = 1; numOfY = 1;
    elseif numOfGraph == 2
        numOfX = 2; numOfY = 1;
    elseif numOfGraph == 3
        numOfX = 3; numOfY = 1;
    elseif numOfGraph == 4
        numOfX = 2; numOfY = 2;
    elseif numOfGraph == 5 | numOfGraph == 6
        numOfX = 3; numOfY = 2;
    elseif numOfGraph == 8 | numOfGraph == 7
        numOfX = 4; numOfY = 2;
    elseif numOfGraph == 9 
        numOfX = 3; numOfY = 3;
    elseif numOfGraph == 10
        numOfX = 5; numOfY = 2;
    elseif numOfGraph == 11 | numOfGraph == 12
        numOfX = 4; numOfY = 3;
    elseif numOfGraph == 13 | numOfGraph == 14 | numOfGraph == 15
        numOfX = 5; numOfY = 3;
    elseif numOfGraph == 16 
        numOfX = 4; numOfY = 4;
    elseif numOfGraph == 17 | numOfGraph == 18 
        numOfX = 6; numOfY = 3;
    elseif numOfGraph == 19 | numOfGraph == 20 
        numOfX = 5; numOfY = 4;
    elseif numOfGraph == 21 
        numOfX = 7; numOfY = 3;
    elseif numOfGraph == 22 | numOfGraph == 23 | numOfGraph == 24
        numOfX = 6; numOfY = 4;
    elseif numOfGraph == 25
        numOfX = 5; numOfY = 5;
    elseif numOfGraph == 26 | numOfGraph == 27 | numOfGraph == 28
        numOfX = 7; numOfY = 4;
    elseif numOfGraph == 29 | numOfGraph == 30 
        numOfX = 6; numOfY = 5;
    else
        error('To many graphs (more than 30)');
    end
    
function legends(agentNames, location)
    if nargin==1
        location = 0;
    end

    if size(agentNames, 1) == 1
        legend(agentNames(1,:), location);
    elseif size(agentNames, 1) == 2
        legend(agentNames(1,:), agentNames(2,:), location);
    elseif size(agentNames, 1) == 3
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), location);
    elseif size(agentNames, 1) == 4
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), location);
    elseif size(agentNames, 1) == 5
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), location);
     elseif size(agentNames, 1) == 6
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
             agentNames(6,:), location);
     elseif size(agentNames, 1) == 7
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
            agentNames(6,:), agentNames(7,:), location);
    else
        error('to many legends');
    end
    