function plotMultBeta(kOpt, V, param, beta,  folder)
% function plotMultBeta(kOpt, V, param, beta,  folder)
%
% plots graphs to the Life Cycle of companies

param = checkParam(param);
maxStates = size(kOpt,2)-1;
r = 1./beta - 1;

state = 2; % state of the recession

lw = 2;

if nargin<4
    error('not enough input arguments');
end


if nargin<5
    folder = 'lastRun';
end

% if 7== exist('f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\','dir')
%     defaultPath = 'f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\';
% elseif 7== exist('C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\','dir')
%     defaultPath = 'C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\';
% else
%    mkdir(cd, 'Graphs');
    defaultPath = strcat(cd, '\Graphs\');
% end

path = strcat(defaultPath, folder, '\');


lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes)

try
    save(strcat(path,'BetaVars') , 'kOpt', 'V', 'param', 'beta');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    save(strcat(path,'BetaVars') , 'kOpt', 'V', 'param', 'beta');
end

  

% Optimal Capital (Next Period)
% ------------------------------

figure;
 
maxK = squeeze(max(kOpt, [], 1));

for ii = 1:numel(maxK(2,:))
   maxK(3,ii) = interp1(kOpt(:,1,ii), kOpt(:,1+state,ii) ,maxK(2,ii) );
end

kToSurvive = NaN(numel(beta), 1);

for ii = 1:numel(beta)
    param.beta = beta(ii);  
    start =  [0, ( (1+param.delta)/param.gamma/param.alpha)^(1/(param.alpha-1))];
    temp = fSupportable(start, state, param);
    if temp(2) <0
        kToSurvive(ii) = NaN; 
    else
        kToSurvive(ii) = fzero(@(k) fSupportable(k, state, param), start); 
    end
end

subplot(2,1,1);
plot(r, [maxK(2,:)', kToSurvive], 'LineWidth', lw);
title('Optimal Capital Stock in the Next Period')
ylabel('s=1')
xlabel('r')
ylim([0, 1.1*max(max(max(maxK(2:3,:)))) ] );

subplot(2,1,2);
plot(r, [maxK(3,:)', kToSurvive], 'LineWidth', lw);
ylabel('s=2')
xlabel('r')
ylim([0, 1.1*max(max(max(maxK(2:3,:)))) ] );

try
    saveas(gcf, strcat(path,'BetaOptK') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'BetaOptK') , 'epsc');
end

% Optimal Dividend (Actual Period)
% ---------------------------------


d = NaN(numel(beta), numel(param.s)+1);
dOpt = kOpt;
for ii = 1:numel(beta)
    param.beta = beta(ii);   
    [dStar, fStar] = calcDiv(kOpt(:,:,ii), param);
        dOpt(:,:,ii) = dStar;
    d(ii, 1) = interp1(dStar(:,1),dStar(:,2),maxK(2,ii));
    d(ii, 3) = interp1(dStar(:,1),dStar(:,3),maxK(3,ii));
    %d(ii, 3) = interp1(dStar(:,1),dStar(:,3), min(maxK(2:3,ii), [], 2));
    d(ii, 2) = interp1(dStar(:,1),dStar(:,3),maxK(2,ii));
end

figure;
subplot(3,1,1)
plot(r, d(:,1), 'LineWidth', lw)
title('Dividend Policy')
ylabel('s=1')
xlabel('r')
ylim([0, 1.1*max(max(max(d(:,[1,3])))) ] );

subplot(3,1,2)
plot(r, d(:,2), 'LineWidth', lw)
ylabel('s=2 with capital from s=1')
xlabel('r')

subplot(3,1,3)
plot(r, d(:,3), 'LineWidth', lw)
ylabel('s=2')
xlabel('r')
ylim([0, 1.1*max(max(max(d(:,[1,3])))) ] );
    
saveas(gcf, strcat(path,'BetaOptD') , 'epsc');

% Dividends and capital starting from Optimal k
% -----------------------------------------------

tt = [0 1 2 5];
tMax = max(tt);
dt = NaN(numel(beta), tMax+1);
kt = NaN(numel(beta), tMax+1);
dt(:,1) = d(:,1);
kt(:,1) = maxK(2,:)';
for t = 1:tMax
    for ii = 1:numel(beta)
        param.beta = beta(ii);
        kt(ii, t+1) = interp1(kOpt(:,1,ii), kOpt(:,1+state,ii), kt(ii,t));
        dt(ii, t+1) = f(kt(ii,t), param.s(state), param) - kt(ii, t+1);
    end
end

% Plot kt
%--------

ktMax = 1.05*max(max(kt));

figure;
for ii = 1:numel(tt)

    subplot(ceil(numel(tt)/2), 2, ii)
    plot(r, [kt(:,tt(ii)+1), kToSurvive], 'LineWidth', lw)    
    ylabel(strcat('t=', num2str(tt(ii))));
    
    if ii==numel(tt) | ii == numel(tt)-1
        xlabel('r')
    end
    ylim([0, ktMax]);

end
saveas(gcf, strcat(path,'BetaKt') , 'epsc');

% Plot dt
%--------
figure;
dtMax = 1.05*max(max(dt));
for ii = 1:numel(tt)
    subplot(ceil(numel(tt)/2), 2, ii)
    plot(r, dt(:,tt(ii)+1), 'LineWidth', lw)
    ylabel(strcat('t=', num2str(tt(ii))));
    if ii == numel(tt) | ii == numel(tt)-1
        xlabel('r')
    end
    ylim([0, dtMax]);

end
saveas(gcf, strcat(path,'BetaDt') , 'epsc');

% NPV depending on S and K
% ------------------------------

figure;
index = V(:,1,1)<= 1.25 * max(max(max(kOpt(:, 2:end, :))));
[X, Y] = ndgrid(V(index,1,1), r');
for ii = 1:(size(V, 2)-1)
    subplot(1,(size(V, 2)-1), ii);
    [C,h] = contour(X, Y, squeeze(V(index,ii+1,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k');
    ylabel('r');
end
saveas(gcf, strcat(path,'BetaNPVsk') , 'epsc');

% kNext
% -----

figure;
[X, Y] = ndgrid(V(index,1,1), r');
for ii = 1:(size(kOpt, 2)-1)
    subplot(1,(size(kOpt, 2)-1), ii);
    [C,h] = contour(X, Y, squeeze(kOpt(index,ii+1,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k');
    ylabel('r');
end
saveas(gcf, strcat(path,'BetakNext') , 'epsc');


% DivOpt
% ------

figure;
[X, Y] = ndgrid(V(index,1,1), r');
for ii = 1:(size(kOpt, 2)-1)
    subplot(1,(size(dOpt, 2)-1), ii);
    [C,h] = contour(X, Y, squeeze(dOpt(index,ii+1,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k');
    ylabel('r');
end
saveas(gcf, strcat(path,'BetadOpt') , 'epsc');



% Probability to die
% ------------------
T = 5;
index = kOpt(:,1,1)<=1.1*max(max(max(kOpt(:,2:end,:))));
showPeriods = [1 2 T];
pDD = NaN(numel(beta), numel(showPeriods), maxStates);
pD = NaN(sum(index), maxStates, T, numel(beta)); 
gK = NaN(sum(index), maxStates, T, numel(beta)); 
gV = NaN(sum(index), maxStates, T, numel(beta)); 
vola = NaN(sum(index), maxStates, T, numel(beta)); 
for ii = 1:numel(beta)
    param.beta = beta(ii);    
    [pDie, kAbs, kRel, kGrowth, Vabs, Vrel, Vgrowth, VabsVola, VrelVola, Vvola]= ...
        calcVars(kOpt(:,:,ii), V(:,:,ii), param, T, index);
    
    pDie(isnan(pDie)) = 1;
    pD(:,:,:,ii) = pDie;
    gK(:,:,:,ii) = kRel;
    gV(:,:,:,ii) = Vrel;
    vola(:,:,:,ii) = VrelVola;

    for ss = 1:maxStates
        for jj = 1:numel(showPeriods)
            pDD(ii, jj, ss) = interp1(kOpt(index,1,1),pDie(:,ss, showPeriods(jj)),maxK(2,ii));
        end
    end

end


figure;
for jj = 1:numel(showPeriods)
   for ss = 1:maxStates
       subplot(numel(showPeriods), maxStates, (jj-1) * maxStates + ss);
       plot(beta, pDD(:, jj, ss), 'LineWidth', lw);
       ylim([-.05, 1.05]);
       if ss==1
            ylabel(strcat('t=', num2str(showPeriods(jj))));
       end

       if jj == 1
           title(strcat('State ', num2str(ss)));
       end
       
       if jj == numel(showPeriods);
           xlabel('\beta');
       end
   end
end
saveas(gcf, strcat(path,'BetapDD') , 'epsc');


% pD
% ------

figure;
[X, Y] = ndgrid(V(index,1,1), r');
for ii = 1:(size(kOpt, 2)-1)
    subplot(1,(size(kOpt, 2)-1), ii);
    [C,h] = contour(X, Y, squeeze(pD(:,ii,T,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k');
    ylabel('r');
end
saveas(gcf, strcat(path,'BetapD') , 'epsc');


% Growth of Capital
% -------------------

figure;
[X, Y] = ndgrid(V(index,1,1), r');
for ii = 1:(size(kOpt, 2)-1)
    subplot(1,(size(kOpt, 2)-1), ii);
    [C,h] = contour(X, Y, squeeze(gK(:,ii,T,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k');
    ylabel('r');
end
saveas(gcf, strcat(path,'BetagK') , 'epsc');


% Volatility
% -------------------

figure;
[X, Y] = ndgrid(V(index,1,1), r');
for ii = 1:(size(kOpt, 2)-1)
    subplot(1,(size(kOpt, 2)-1), ii);
    [C,h] = contour(X, Y, squeeze(vola(:,ii,T,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k');
    ylabel('r');
end
saveas(gcf, strcat(path,'BetaVola') , 'epsc');


% Growth of Value
% -------------------

figure;
[X, Y] = ndgrid(V(index,1,1), r');
for ii = 1:(size(kOpt, 2)-1)
    subplot(1,(size(kOpt, 2)-1), ii);
    [C,h] = contour(X, Y, squeeze(gV(:,ii,T,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k');
    ylabel('r');
end
saveas(gcf, strcat(path,'BetagV') , 'epsc');




function gain = fSupportable(k, state, param)
    % calculates the gain given the parameters
    % in the bad state
    
    gain = f(k, 0, param)- param.s(state) - k;

function [numOfX, numOfY] = positionsInSubPlot(numOfGraph)
    % determines position in a subplot out of the number of graphs
    
    numOfGraph = ceil(numOfGraph);
    if numOfGraph<1
        error('numOfGraph must be at least 1')
    elseif numOfGraph == 1
        numOfX = 1; numOfY = 1;
    elseif numOfGraph == 2
        numOfX = 2; numOfY = 1;
    elseif numOfGraph == 3
        numOfX = 3; numOfY = 1;
    elseif numOfGraph == 4
        numOfX = 2; numOfY = 2;
    elseif numOfGraph == 5 | numOfGraph == 6
        numOfX = 3; numOfY = 2;
    elseif numOfGraph == 8 | numOfGraph == 7
        numOfX = 4; numOfY = 2;
    elseif numOfGraph == 9 
        numOfX = 3; numOfY = 3;
    elseif numOfGraph == 10
        numOfX = 5; numOfY = 2;
    elseif numOfGraph == 11 | numOfGraph == 12
        numOfX = 4; numOfY = 3;
    elseif numOfGraph == 13 | numOfGraph == 14 | numOfGraph == 15
        numOfX = 5; numOfY = 3;
    elseif numOfGraph == 16 
        numOfX = 4; numOfY = 4;
    elseif numOfGraph == 17 | numOfGraph == 18 
        numOfX = 6; numOfY = 3;
    elseif numOfGraph == 19 | numOfGraph == 20 
        numOfX = 5; numOfY = 4;
    elseif numOfGraph == 21 
        numOfX = 7; numOfY = 3;
    elseif numOfGraph == 22 | numOfGraph == 23 | numOfGraph == 24
        numOfX = 6; numOfY = 4;
    elseif numOfGraph == 25
        numOfX = 5; numOfY = 5;
    elseif numOfGraph == 26 | numOfGraph == 27 | numOfGraph == 28
        numOfX = 7; numOfY = 4;
    elseif numOfGraph == 29 | numOfGraph == 30 
        numOfX = 6; numOfY = 5;
    else
        error('To many graphs (more than 30)');
    end
    
function legends(agentNames, location)
    if nargin==1
        location = 0;
    end

    if size(agentNames, 1) == 1
        legend(agentNames(1,:), location);
    elseif size(agentNames, 1) == 2
        legend(agentNames(1,:), agentNames(2,:), location);
    elseif size(agentNames, 1) == 3
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), location);
    elseif size(agentNames, 1) == 4
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), location);
    elseif size(agentNames, 1) == 5
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), location);
     elseif size(agentNames, 1) == 6
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
             agentNames(6,:), location);
     elseif size(agentNames, 1) == 7
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
            agentNames(6,:), agentNames(7,:), location);
    else
        error('to many legends');
    end
    