function plotSpecial(kOpt, V, param, index, folder)
% function plotSpecial(kOpt, V, param, index, folder)
%
% plots graphs to the Life Cycle of companies

param = checkParam(param);
maxStates = size(kOpt,2)-1;

lw = 2;

if nargin<3
    error('not enough input arguments');
end

if nargin>=4
    kOpt = kOpt(index,:);
    V    = V(index,:);
end

if nargin<5
    folder = 'lastRun';
end

% if 7== exist('f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\','dir')
%     defaultPath = 'f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\';
% elseif 7== exist('C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\','dir')
%     defaultPath = 'C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\';
% else
    mkdir(cd, 'Graphs');
    defaultPath = strcat(cd, '\Graphs\');
% end

path = strcat(defaultPath, folder, '\');


lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes)


% % relative Weahlth with confidence Intervals
% figure;
% relWealth = m.wealth ./ repmat(sum(m.wealth, 2), 1, numOfInvestors);
% aMean = mean(relWealth, 3);
% aLowQ = quantileDim(relWealth, alpha/2, 3);
% aHighQ = quantileDim(relWealth, 1-alpha/2, 3);
% plot(index, aMean(index,:,:), index, aLowQ(index,:,:), ':', ...
%     index, aHighQ(index,:,:), ':');
% axis([min(index), max(index), 0, 1]);
% title('Relative Wealth');
% xlabel(cat(2, 'Mean and  ', num2str(100*(1-alpha)), '% Confidence Interval'));
% legends(m.struct.strategies(m.struct.investors,:));


% Brutto Gain
% ------------------------

figure;
 
[dStar, fStar] = calcDiv(kOpt, param);

plot(fStar(:,1), fStar(:,2:end)-repmat(fStar(:,1), 1, size(fStar, 2)-1), 'LineWidth', lw)
title('Brutto Gain');
ylabel('f(k_t, s_t)-k_t');
xlabel('k_t');

try
    saveas(gcf, strcat(path,'BruttoGain') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'BruttoGain') , 'epsc');
end


% Plot V
%--------

[temp, Vext] = VnKlausStyle(V, param);
Vext(Vext<0) = NaN;

[X, Y] = meshgrid(fStar(:,1), fStar(:,1));

figure;
for ii = 1:size(Vext, 2)
    subplot(1,size(Vext, 2), ii);
    [C,h] = contour(X, Y, squeeze(Vext(:,ii,:)));
    set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2);
    colormap cool
    title(strcat('State ', num2str(ii)));
    xlabel('k Next Period');
    ylabel('k');
end

try
    saveas(gcf, strcat(path,'VtoOpt') , 'epsc');
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    saveas(gcf, strcat(path,'VtoOpt') , 'epsc');
end

function [numOfX, numOfY] = positionsInSubPlot(numOfGraph)
    % determines position in a subplot out of the number of graphs
    
    numOfGraph = ceil(numOfGraph);
    if numOfGraph<1
        error('numOfGraph must be at least 1')
    elseif numOfGraph == 1
        numOfX = 1; numOfY = 1;
    elseif numOfGraph == 2
        numOfX = 2; numOfY = 1;
    elseif numOfGraph == 3
        numOfX = 3; numOfY = 1;
    elseif numOfGraph == 4
        numOfX = 2; numOfY = 2;
    elseif numOfGraph == 5 | numOfGraph == 6
        numOfX = 3; numOfY = 2;
    elseif numOfGraph == 8 | numOfGraph == 7
        numOfX = 4; numOfY = 2;
    elseif numOfGraph == 9 
        numOfX = 3; numOfY = 3;
    elseif numOfGraph == 10
        numOfX = 5; numOfY = 2;
    elseif numOfGraph == 11 | numOfGraph == 12
        numOfX = 4; numOfY = 3;
    elseif numOfGraph == 13 | numOfGraph == 14 | numOfGraph == 15
        numOfX = 5; numOfY = 3;
    elseif numOfGraph == 16 
        numOfX = 4; numOfY = 4;
    elseif numOfGraph == 17 | numOfGraph == 18 
        numOfX = 6; numOfY = 3;
    elseif numOfGraph == 19 | numOfGraph == 20 
        numOfX = 5; numOfY = 4;
    elseif numOfGraph == 21 
        numOfX = 7; numOfY = 3;
    elseif numOfGraph == 22 | numOfGraph == 23 | numOfGraph == 24
        numOfX = 6; numOfY = 4;
    elseif numOfGraph == 25
        numOfX = 5; numOfY = 5;
    elseif numOfGraph == 26 | numOfGraph == 27 | numOfGraph == 28
        numOfX = 7; numOfY = 4;
    elseif numOfGraph == 29 | numOfGraph == 30 
        numOfX = 6; numOfY = 5;
    else
        error('To many graphs (more than 30)');
    end
    
function legends(agentNames, location)
    if nargin==1
        location = 0;
    end

    if size(agentNames, 1) == 1
        legend(agentNames(1,:), location);
    elseif size(agentNames, 1) == 2
        legend(agentNames(1,:), agentNames(2,:), location);
    elseif size(agentNames, 1) == 3
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), location);
    elseif size(agentNames, 1) == 4
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), location);
    elseif size(agentNames, 1) == 5
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), location);
     elseif size(agentNames, 1) == 6
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
             agentNames(6,:), location);
     elseif size(agentNames, 1) == 7
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
            agentNames(6,:), agentNames(7,:), location);
    else
        error('to many legends');
    end
    