function plotYSingle(kOpt, V, param, index, folder)
% function plotLC(kOptY, Vy, param, index, folder)
%
% plots graphs to the Life Cycle of companies

param = checkParam(param);
maxStates = size(kOpt,2)-1;



lw = 1;
xLimits = [min(V(index,1)) max(V(index,1))]

if nargin<3
    error('not enough input arguments');
end

if nargin>=4
    kOpt = kOpt(index,:);
    V    = V(index,:);
end

if nargin<5
    folder = 'lastRun';
end

% if 7== exist('f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\','dir')
%     defaultPath = 'f:\Uni\SS 06\evoModel\OutputFromHome\Graphs\';
% elseif 7== exist('C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\','dir')
%     defaultPath = 'C:\Documents and Settings\Urs\My Documents\evoModel\OutputFromOffice\Graphs\';
% else
    mkdir(cd, 'Graphs');
    defaultPath = strcat(cd, '\Graphs\');
% end

path = strcat(defaultPath, folder, '\');

filename=strcat(path,'data.xlsx');

if (size(kOpt, 2)==3)
    colnames = {'y', 'S=1', 'S=2'};
elseif (size(kOpt, 2)==4)
    colnames = {'y', 'S=1', 'S=2', 'S=3'};
elseif (size(kOpt, 2)==5)
    colnames = {'y', 'S=1', 'S=2', 'S=3', 'S=4'};
else
    colnames = {'y', 'S=1', 'S=2', 'S=3', 'S=4', 'S=5', '...'};
end

lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes)



% Overview over functions
% ------------------------

figure;
plot(V(:,1), V(:,2:end), 'LineWidth', lw)
% title('(a) Value Function');
% ylabel('V');
xlabel('y');
xlim(xLimits)

saveToTXT(strcat(path,'LcVy', '.txt'), V, 'yV');
saveToXLSX(filename,'LcVy', V, colnames, {'Value Function'});
try
    exportfig(gcf,strcat(path, 'LcVy', '.eps'), 'width',12, 'fontmode','fixed', 'fontsize',6);
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    exportfig(gcf,strcat(path, 'LcVy', '.eps'), 'width',12, 'fontmode','fixed', 'fontsize',6);
end



plot(kOpt(:,1), kOpt(:,2:end), 'LineWidth', lw)
% title('(b) Optimal Capital in the Next Period');
% ylabel('k^*_{t+1}');
xlabel('y');
xlim(xLimits)

saveToTXT(strcat(path,'LcKy', '.txt'), kOpt, 'yV');
saveToXLSX(filename,'LcKy', kOpt, colnames, {'Optimal capital in next period'});
try
    exportfig(gcf,strcat(path, 'LcKy', '.eps'), 'width',6, 'fontmode','fixed', 'fontsize',6);
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    exportfig(gcf,strcat(path, 'LcKy', '.eps'), 'width',6, 'fontmode','fixed', 'fontsize',6);
end

nS = size(kOpt,2)-1;
nG = size(kOpt,1);
dStar  = [kOpt(:,1), repmat(kOpt(:,1),1,nS)- kOpt(:,2:end)];
s = repmat(param.s', nG,1);
fStar = [kOpt(:,1), f(repmat(kOpt(:,1),1,nS), s, param)];
%[dStar, fStar] = calcDivY(kOpt, param);
xlim(xLimits)


plot(dStar(:,1), dStar(:,2:end), 'LineWidth', lw)
% title('(c) Optimal Dividend Payment');
% ylabel('d^*');
xlabel('y');
xlim(xLimits) 
set(gca,'YTick',[0:1:3])
saveToTXT(strcat(path,'LcDy', '.txt'), dStar, 'yV')
saveToXLSX(filename,'LcDy', dStar, colnames, {'Optimal Dividends'});

try
    exportfig(gcf,strcat(path, 'LcDy', '.eps'), 'width',6, 'fontmode','fixed', 'fontsize',6);
catch %if there is no directory create one
    mkdir(defaultPath, folder)
    exportfig(gcf,strcat(path, 'LcDy', '.eps'), 'width',6, 'fontmode','fixed', 'fontsize',6);
end





function [numOfX, numOfY] = positionsInSubPlot(numOfGraph)
    % determines position in a subplot out of the number of graphs
    
    numOfGraph = ceil(numOfGraph);
    if numOfGraph<1
        error('numOfGraph must be at least 1')
    elseif numOfGraph == 1
        numOfX = 1; numOfY = 1;
    elseif numOfGraph == 2
        numOfX = 2; numOfY = 1;
    elseif numOfGraph == 3
        numOfX = 3; numOfY = 1;
    elseif numOfGraph == 4
        numOfX = 2; numOfY = 2;
    elseif numOfGraph == 5 | numOfGraph == 6
        numOfX = 3; numOfY = 2;
    elseif numOfGraph == 8 | numOfGraph == 7
        numOfX = 4; numOfY = 2;
    elseif numOfGraph == 9 
        numOfX = 3; numOfY = 3;
    elseif numOfGraph == 10
        numOfX = 5; numOfY = 2;
    elseif numOfGraph == 11 | numOfGraph == 12
        numOfX = 4; numOfY = 3;
    elseif numOfGraph == 13 | numOfGraph == 14 | numOfGraph == 15
        numOfX = 5; numOfY = 3;
    elseif numOfGraph == 16 
        numOfX = 4; numOfY = 4;
    elseif numOfGraph == 17 | numOfGraph == 18 
        numOfX = 6; numOfY = 3;
    elseif numOfGraph == 19 | numOfGraph == 20 
        numOfX = 5; numOfY = 4;
    elseif numOfGraph == 21 
        numOfX = 7; numOfY = 3;
    elseif numOfGraph == 22 | numOfGraph == 23 | numOfGraph == 24
        numOfX = 6; numOfY = 4;
    elseif numOfGraph == 25
        numOfX = 5; numOfY = 5;
    elseif numOfGraph == 26 | numOfGraph == 27 | numOfGraph == 28
        numOfX = 7; numOfY = 4;
    elseif numOfGraph == 29 | numOfGraph == 30 
        numOfX = 6; numOfY = 5;
    else
        error('To many graphs (more than 30)');
    end
    
function legends(agentNames, location)
    if nargin==1
        location = 0;
    end

    if size(agentNames, 1) == 1
        legend(agentNames(1,:), location);
    elseif size(agentNames, 1) == 2
        legend(agentNames(1,:), agentNames(2,:), location);
    elseif size(agentNames, 1) == 3
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), location);
    elseif size(agentNames, 1) == 4
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), location);
    elseif size(agentNames, 1) == 5
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), location);
     elseif size(agentNames, 1) == 6
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
             agentNames(6,:), location);
     elseif size(agentNames, 1) == 7
        legend(agentNames(1,:), agentNames(2,:),...
            agentNames(3,:), agentNames(4,:), agentNames(5,:), ...
            agentNames(6,:), agentNames(7,:), location);
    else
        error('to many legends');
    end
    