function [k, div, V] = simuPath(k, state, T, kOpt, Vopt, param)
% function [k, div, V] = simuPath(k, state, T, kOpt, Vopt, param)
%
% simulates T periods with starting parameter k
% and starting state

    % Consistency Checks
    % ------------------
    
    if nargin~=6
        error('Wrong number of input arguments');
    end
    
    T = fix(T(1));
    if T<1
        error('T must be at least 1');
    end
    
    k = vec(k);
    state = vec(state);
    
    if any(size(k) ~=size(state))
        error('k and state need the same size');
    end   
    
    param = checkParam(param); % checks parameters of param on consitencyx
    
   
    p = param.p;
    s = param.s;
    nOfStates = numel(s);
    cumP = cumsum(p);
    cumP = [zeros(1, nOfStates);cumP];
    
    if fix(state) ~= state | any(state>nOfStates)
        error('state has to be an integer or the state is bigger than the number of existing states');
    end
    
    if any(size(p)~= (size(kOpt, 2)-1))
        error('p or Vpre have a wrong dimension');
    end
    
    
    % Initialization of the results
    % ------------------------------
        
    k     = [k, NaN(size(state,1), T)];
    div   = NaN(size(k));
    V     = NaN(size(k));
    
    
    % Simulation of states
    % ---------------------
    stateSimu = state;
    state = NaN(numel(state), T+1);
    state(:,1) = stateSimu;
    
    for t = 2:(T+1)        
        stateSimu = rand(1, size(state,1));
    
        for ii = nOfStates:-1:1
             state(stateSimu>= cumP(ii,state(:,t-1)) & ...
                 stateSimu<= cumP(ii+1,state(:,t-1)), t) = ii;      
        end 
    end
    
    
    % Calculate Results
    % ------------------
    
    for ii = 1:(T+1)
        fAkt = f(k(:,ii), s(state(:,ii)), param );
        kNext = calcKopt(k(:,ii), state(:,ii), kOpt);
        div(:,ii) = fAkt-kNext;
        
        for ss = 1:nOfStates
            temp = state(:,ii)==ss;
            if sum(temp)>0
                V(temp,ii) = interp1(Vopt(:,1), Vopt(:,ss+1), k(temp,ii), 'linear');
            end
        end
        
        if ii<=T
            k(:,ii+1) = kNext;
        end
    end
    
    
    
    