% Cobb Douglas Standard Case
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% mex VyOpt.c
% mex VyOptUpper.c
global ExportToExcel;
ExportToExcel=false;

param.case  = 'CobbDouglas';
param.beta  = 0.95;
param.alpha = 0.8;
param.gamma = 2;
param.delta = 0.1;
param.plot = false;
param.T = 5;


param.s = [0, 2];
param.p = [0.5, 0.5;
    0.5, 0.5];

folder = 'CD1'
file   = ['Graphs/' folder, '/CD1iidEqualProb'];
mkdir(cd,file)
fileTxt = [file '.txt'];
fileMat = [file '.mat']

param.scale = 1;
param.checked = false;

gridSize = 1/1000
gridPoints = 0:gridSize:20;
IterMax    = 500
tol = gridSize/10;

[Vk, kOpt, Vy, kOptY]   = Viterate(param, gridPoints, tol, IterMax);

maxK  = 1.8*max(max(kOpt(:,2:end)));
index = kOpt(:,1)<=8;
save(fileMat, 'Vk', 'Vy', 'kOpt', 'kOptY', 'param', 'tol', 'maxK', 'index', 'gridPoints', 'IterMax', 'folder')

[pDie, yGrowth, Vgrowth, Vvola, ExpRet, VolaRet, ExpROI, VolaROI]=  calcVarsY(kOptY, Vy, param, param.T, index);
save(fileMat, 'Vk', 'Vy', 'kOpt', 'kOptY', 'param', 'tol', 'maxK', 'index', 'gridPoints', 'IterMax', 'pDie', 'yGrowth', 'Vgrowth', 'Vvola', 'ExpRet', 'VolaRet', 'ExpROI', 'VolaROI', 'folder')



% Save Parameter to XLS
%%%%%%%%%%%%%%%%%%%%%%%%%

filename=strcat(cd, '\Graphs\', folder, '\','data.xlsx');
sheetname = 'param';

if ExportToExcel
    xlswrite(filename, {'alpha'}, sheetname, 'A1');
    xlswrite(filename, param.alpha, sheetname, 'A2');
    xlswrite(filename, {'beta'}, sheetname, 'B1');
    xlswrite(filename, param.beta, sheetname, 'B2');
    xlswrite(filename, {'gamma'}, sheetname, 'C1');
    xlswrite(filename, param.gamma, sheetname, 'C2');
    xlswrite(filename, {'delta'}, sheetname, 'D1');
    xlswrite(filename, param.delta, sheetname, 'D2');
    xlswrite(filename, {'Shocks'}, sheetname, 'A4');
    xlswrite(filename, param.s, sheetname, 'A5');
    xlswrite(filename, {'P'}, sheetname, 'A7');
    xlswrite(filename, param.p, sheetname, 'A8');
end




% Charts
%%%%%%%%%%

showPeriods = [1];

lw=1;
lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes);

limits=[0.1,max(max(kOpt(:,2:end)))];
retIndex = kOptY(:,1)<= limits(2)   & kOptY(:, 1)>=limits(1);
retIndex = retIndex(index);


yLarger=kOptY(:, 1)>0.1;
yLarger = yLarger(index);


% Standard Plots
plotY(kOptY, Vy, param, index, folder)
plotYSingle(kOptY, Vy, param, index, folder)


% TobinQ
plot(kOptY(yLarger, 1), Vy(yLarger,2)./kOptY(yLarger, 1), 'LineWidth', lw)
xlabel('y');
xlim([0.1, 7]);
ylim([2, 6])
filename=strcat(cd, '\Graphs\', folder, '\','TobinQ.eps');
exportfig(gcf,filename, 'width',12, 'fontmode','fixed', 'fontsize',6);

% MuSigma
VySmall = Vy(index,:);
plot(VySmall(retIndex,2), squeeze(VolaRet(retIndex,1,5)), 'LineWidth', lw)
xlabel('Market Value');
ylabel('Volatility of Return');
ylim([0, 0.65])
filename=strcat(cd, '\Graphs\', folder, '\','ValueVola5.eps');
exportfig(gcf,filename, 'width',6, 'height',5, 'fontmode','fixed', 'fontsize',6);
colnames = {'V', 'vola'};
saveToTXT(strcat(cd, '\Graphs\', folder, '\','Leverage.txt'), [VySmall(retIndex,2), squeeze(VolaRet(retIndex,1,5))], 'Vo');
saveToXLSX(strcat(cd, '\Graphs\', folder, '\','data.xlsx'),'Leverage', [VySmall(retIndex,2), squeeze(VolaRet(retIndex,1,5))], colnames, {'Leverage Effect'});


% Value Premium
plot(VySmall(retIndex,2)./VySmall(retIndex,1), squeeze(ExpRet(retIndex,1,5)), 'LineWidth', lw)
xlabel('Market-to-Book-Ratio');
ylabel('Expected Return');
xlim([2.5 6.5]), ylim([0 1.1]);
filename=strcat(cd, '\Graphs\', folder, '\','BookToMarketMu5.eps');
exportfig(gcf,filename, 'width',6, 'height',5, 'fontmode','fixed', 'fontsize',6);
colnames = {'V', 'BTM'};
saveToTXT(strcat(cd, '\Graphs\', folder, '\','BTMtoRet.txt'), [VySmall(retIndex,2)./VySmall(retIndex,1), squeeze(ExpRet(retIndex,1,5))], 'Vo');
saveToXLSX(strcat(cd, '\Graphs\', folder, '\','data.xlsx'),'BTMtoRet', [VySmall(retIndex,2)./VySmall(retIndex,1), squeeze(ExpRet(retIndex,1,5))], colnames, {'Leverage Effect'});



% Various Calculations of Figures in the Text
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Calc OptCap
p0 = cumsum(param.p);

kBench = 1./(p0.*param.beta) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1));

fBench = [param.s' ,param.gamma.*kBench.^param.alpha - param.delta.*kBench];

kBench

max(kOpt(:,2:end))


% Range k 
kBar = (param.alpha*param.gamma/param.delta)^(1/(1-param.alpha))
fBar = f(kBar, 0, param)
kPreserving = (param.gamma/(1+param.delta)) ^ (1/(1-param.alpha))
fPreserving = f(kPreserving, 0, param)
kMarginal   = (param.gamma*param.alpha/(1+param.delta) )^(1/(1-param.alpha))
fMarginal = f(kMarginal, 0, param)

kInfSurvival = ((1+param.delta)/param.gamma/param.alpha)^(1/(param.alpha-1))
fInfSurvival = f(kInfSurvival, 0, param)
fInfSurvival - kInfSurvival

% Value
yValue = 0:0.5:6
Value = Vy(1/gridSize*yValue+1,:)

tempK = kBench(2,1)
tempF = f(tempK, param.s, param)
ValueBenchIID = sum(param.p(:,1)'.*(tempF-tempK))/(1-param.beta)
ValueBenchIID = yValue-tempK + param.beta *ValueBenchIID

ValueTable = [yValue' ValueBenchIID' Value(:,2) 100-Value(:,2)./ValueBenchIID'*100]


% Eat up capital
kStart =0.905
kStart = [0:20; repmat(kStart, 1,21)];
for ii = 1:20
    kStart(2,ii+1) = f(kStart(2,ii), param.s(2) ,param);
    
end

kStart


% Table Default and Growth
tempInd = [501 1001 2001 3001 4001 5001 6001]
tabP  = [squeeze(pDie(tempInd, 1, [1 5])), squeeze(yGrowth(tempInd, 1, [1]))]'


% How much capital is needed to exactly survive one period
kSurvive1 = fzero(@(x) f(x, 0, param)-2, 1) 
