% i.i.d. case with multiple shocks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% mex VyOpt.c
global ExportToExcel
ExportToExcel=true

param.T=5
param.case  = 'CobbDouglas';
param.beta  = 0.95;
param.alpha = 0.8;
param.gamma = 2;
param.delta = 0.1;
param.plot = false;

temp = 1.5
param.s = [0 2 6];
pp = 0.455;
p1 = pp;
p2 = pp;
p3 = 1-p1-p2;
param.p = [p1 p1 p1; 
           p2 p2 p2;
           p3 p3 p3];


folder = 'CD10'
file   = ['Graphs/' folder, '/CD10 3States']
mkdir(cd,file)
fileTxt = [file '.txt']
fileMat = [file '.mat']

param.scale = 1;
param.checked = false;

gridSize = 1/1000
gridPoints = 0:gridSize:20;
IterMax    = 500
tol = gridSize/10;

[Vk, kOpt, Vy, kOptY]   = Viterate(param, gridPoints, tol, IterMax);
maxK  = 1.8*max(max(kOpt(:,2:end)));
index = kOpt(:,1)<=8;
save(fileMat, 'Vk', 'Vy', 'kOpt', 'kOptY', 'param', 'tol', 'maxK', 'index', 'gridPoints', 'IterMax', 'folder')
 
% Save Params to XLS
%%%%%%%%%%%%%%%%%%%%
 
defaultPath = strcat(cd, '\Graphs\');
filename=strcat(cd, '\Graphs\', folder, '\','data.xlsx');
sheetname = 'param';

if ExportToExcel
    xlswrite(filename, {'alpha'}, sheetname, 'A1');
    xlswrite(filename, param.alpha, sheetname, 'A2');
    xlswrite(filename, {'beta'}, sheetname, 'B1');
    xlswrite(filename, param.beta, sheetname, 'B2');
    xlswrite(filename, {'gamma'}, sheetname, 'C1');
    xlswrite(filename, param.gamma, sheetname, 'C2');
    xlswrite(filename, {'delta'}, sheetname, 'D1');
    xlswrite(filename, param.delta, sheetname, 'D2');
    xlswrite(filename, {'Shocks'}, sheetname, 'A4');
    xlswrite(filename, param.s, sheetname, 'A5');
    xlswrite(filename, {'P'}, sheetname, 'A7');
    xlswrite(filename, param.p, sheetname, 'A8');
end


% Charts
%%%%%%%%%

plotY(kOptY, Vy, param, index, folder)
plotYSingle(kOptY, Vy, param, index, folder)


% Calculations for text
%%%%%%%%%%%%%%%%%%%%%%%


% Optimal Capital
p0 = cumsum(param.p)

kBench = 1./(p0.*param.beta) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))

fBench = [param.s' ,param.gamma.*kBench.^param.alpha - param.delta.*kBench]

kBench

max(kOpt(:,2:end))


% Range k 
kBar = (param.alpha*param.gamma/param.delta)^(1/(1-param.alpha))
fBar = f(kBar, 0, param)
kPreserving = (param.gamma/(param.beta +param.delta)) ^ (1/(1-param.alpha))
fPreserving = f(kPreserving, 0, param)
kMarginal   = (param.gamma*param.alpha/(1+param.delta) )^(1/(1-param.alpha))
fMarginal = f(kMarginal, 0, param)


% Value
yValue = 0:0.5:8
Value = Vy(1/gridSize*yValue+1,:)

tempK = kBench(2,1)
tempF = f(tempK, param.s, param)
ValueBenchIID = sum(param.p(:,1)'.*(tempF-tempK))/(1-param.beta)
ValueBenchIID = yValue-tempK + param.beta *ValueBenchIID

ValueTable = [yValue' ValueBenchIID' Value(:,2) 1-Value(:,2)./ValueBenchIID']
