% CobbDouglas Business Cycles Various Shock Sizes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lw=1;
folder = 'MultShocks'
file   = ['Graphs/' folder, '/CD1iidEqualProb'];
mkdir(cd,file)

param.case  = 'CobbDouglas';
param.beta  = 0.95;
param.alpha = 0.8;
param.gamma = 2;
param.delta = 0.1;
param.plot = false;

param.s = [0, 0.8];
param.p = [0.9, 0.1;
    0.1, 0.9];

param.scale = 1;
param.checked = false;

gridSize = 1/1000
gridPoints = 0:gridSize:20;
n = numel(gridPoints);
step = max(floor(n/2000),1);
index = 1:step:(n*.75);
IterMax    = 500
tol = gridSize/10;

s =[0:0.02:1.4, 1.45: 0.005:2.6, 2.7:0.02:10 ];


V = NaN([numel(index), numel(param.s)+1, numel(s)]);
kOpt = NaN([numel(index), numel(param.s)+1, numel(s)]);
Vy = NaN([numel(index), numel(param.s)+1, numel(s)]);
kOptY = NaN([numel(index), numel(param.s)+1, numel(s)]);

Vtemp = 0; kOptTemp=0; Vytemp = 0; kOptYTemp=0;

try  % close Matlabpool if one is open
    matlabpool close;
catch ME
    disp('Matlabpool has been closed');
end

matlabpool local 3;
poolsize = 24; % Number of Simulations done in a parfor
startindex = 1; %Initialization
tic;
while startindex<=numel(s)
    endindex = startindex+poolsize-1;
    if endindex > numel(s)
        endindex = numel(s)
    end
    
    disp(strcat('Start: Iteration ', num2str(startindex)))
    
    parfor ii = startindex:endindex
    %for ii = startindex:endindex    
        
        paramParFor = param;
        paramParFor.s(2) = s(ii);
        [Vtemp, kOptTemp, Vytemp, kOptTempY] = Viterate(paramParFor, gridPoints, tol, IterMax);
        V(:,:,ii) = Vtemp(index,:);
        kOpt(:,:,ii) = kOptTemp(index,:);
        Vy(:,:,ii) = Vytemp(index,:);
        kOptY(:,:,ii) = kOptTempY(index,:);
    end
    
    save('temp.mat', 'V', 'kOpt', 'Vy', 'kOptY', ...
        'param', 's', 'tol', 'gridPoints', 'IterMax')
    disp(strcat('End: Iteration ', num2str(endindex)))
    usedTime = toc /3600;
    sZwi = strcat('Endshock:  ', num2str(s(endindex)), ...
        '  used time: ',' ', ...
        num2str(usedTime,2), ' h');
    disp(sZwi);
    startindex = endindex +1;
end

gridPoints = gridPoints(index);
%matlabpool close;
save('final.mat', 'V', 'kOpt', 'Vy', 'kOptY', 'param', 's', 'tol', 'gridPoints', 'IterMax', 'index')


plotMultS(kOpt, V, param, s, 'MultShocks')
plotMultSY(kOpt, V, kOptY, Vy, param, gridPoints, s, 'MultShocks')

% Plot of one single shock
selectS = 1.98;

paramSelect = param;
paramSelect.s(2) = selectS;

VSelect = squeeze(V(:,:,abs(s-selectS) < 1e-13));
VySelect = squeeze(V(:,:,abs(s-selectS) < 1e-13));
kOptSelect = squeeze(kOpt(:,:, abs(s-selectS) < 1e-13));
kOptYSelect = squeeze(kOptY(:,:, abs(s-selectS) < 1e-13));


maxK  = 1.5*max(max(kOptSelect(:,2:end)));

index = kOptSelect(:,1)<=maxK;


plotLC(kOptSelect, VSelect, paramSelect, index, 'CDBCmultExample')
plotY(kOptYSelect, VySelect, paramSelect, index, 'CDBCmultExample')

k0 = 1*[1 1 1 1];
state0 = [1 1 1 1];
T = 20;
plotSimuPath(k0, state0, T, kOptSelect, VSelect, paramSelect, 'CDBCmultExample')



% Dividends starting from Optimal k
lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes)

maxK = squeeze(max(kOpt, [], 1));

for ii = 1:numel(maxK(2,:))
   maxK(3,ii) = interp1(kOpt(:,1,ii), kOpt(:,3,ii) ,maxK(2,ii) );
end

kToSurvive = NaN(numel(s), 1);
state = 2;
for ii = 1:numel(s)
    param.s(state) = s(ii);  
    start =  [0, ( (1+param.delta)/param.gamma/param.alpha)^(1/(param.alpha-1))];
    temp = fSupportable(start, state, param);
    if temp(2) <0
        kToSurvive(ii) = NaN; 
    else
        kToSurvive(ii) = fzero(@(k) fSupportable(k, state, param), start); 
    end
end

d = NaN(numel(s), numel(param.s)+1);
dOpt = kOpt;
for ii = 1:numel(s)
    param.s(2) = s(ii);    
    [dStar, fStar] = calcDiv(kOpt(:,:,ii), param);
    dOpt(:,:,ii) = dStar;
    d(ii, 1) = interp1(dStar(:,1),dStar(:,2),maxK(2,ii));
    d(ii, 3) = interp1(dStar(:,1),dStar(:,3),maxK(3,ii));
    %d(ii, 3) = interp1(dStar(:,1),dStar(:,3), min(maxK(2:3,ii), [], 2));
    d(ii, 2) = interp1(dStar(:,1),dStar(:,3),maxK(2,ii));
end

tt = [0 1 2 5];
tMax = max(tt);
dt = NaN(numel(s), tMax+1);
kt = NaN(numel(s), tMax+1);
dt(:,1) = d(:,1);
kt(:,1) = maxK(2,:)';
for t = 1:tMax
    for ii = 1:numel(s)
        kt(ii, t+1) = interp1(kOpt(:,1,ii), kOpt(:,3,ii), kt(ii,t));
        dt(ii, t+1) = f(kt(ii,t), s(ii), param) - kt(ii, t+1);
    end
end

% Plot kt
ktMax = 1.05*max(max(kt));
kKill = 0:(ktMax/1000):ktMax;
sKill = f(kKill, 0, param);

figure;
for ii = 1:numel(tt)

    %subplot(ceil(numel(tt)/2), 2, ii)
    plot(s, [kt(:,tt(ii)+1), kToSurvive], 'LineWidth', lw)    
    %ylabel(strcat('t=', num2str(tt(ii))));
    hold on
    plot(sKill, kKill, ':', 'LineWidth', lw)
    hold off
    
    %if ii==numel(tt) | ii == numel(tt)-1
        xlabel('Shock Size')
    %end
    ylim([0, ktMax]);
    xlim([0, 8])
    %title(letters(ii,:));
    
    
    filename=strcat(cd, '\Graphs\', folder, '\','kt',num2str(tt(ii)),'.eps');
    exportfig(gcf,filename, 'width',6, 'fontmode','fixed', 'fontsize',6);

end

filename=strcat(cd, '\Graphs\', folder, '\','data.xlsx');
sheetname = 'kT';
colnames = {'s', 't=0', 't=1', 't=2', 't=3', 't=4', 't=5'};
saveToXLSX(filename,'kT', [s' kt], colnames, {'Capital over time in dependance of shocks'});
filename=strcat(cd, '\Graphs\', folder, '\','data.txt');
saveToTXT(filename, [s' kToSurvive kt], 'sk');


% Plot dt
figure;
dtMax = 1.05*max(max(dt));
for ii = 1:numel(tt)
    subplot(ceil(numel(tt)/2), 2, ii)
    plot(s, dt(:,tt(ii)+1), 'LineWidth', lw)
    ylabel(strcat('t=', num2str(tt(ii))));
    if ii == numel(tt) | ii == numel(tt)-1
        xlabel('Shock Size')
    end
    ylim([0, dtMax]);
    %title(letters(ii,:));

end
saveas(gcf, strcat(path,'dt') , 'epsc');



% Calc OptCap
p0 = cumsum(param.p)

kBench = 1./(p0.*param.beta) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))

fBench = param.gamma.*kBench.^param.alpha - param.delta.*kBench

[s(1:50:650)', squeeze(max(kOpt(:,2:3,1:50:650)))']'


%sustainable (f(k) = k)
kSust = ((1+param.delta)/param.gamma).^(1./(param.alpha-1))
fSust = f(kSust, 0, param)
