% Central Bank Case: CobbDouglas Business Cycle
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

param.case  = 'CobbDouglas';
param.beta  = [0.95; 0.95];
param.alpha = 0.8;
param.gamma = 2;
param.delta = 0.1;
param.plot = false;

param.s = [0, 2];

param.p = [0.9, 0.1;
    0.1, 0.9];


epsilon = [-0.03, -0.02, -.01, 0, .01 +0.02, 0.03];

param.scale = 1;
param.checked = false;

gridSize = 1/1000 
gridPoints = 0:gridSize:20;
n = numel(gridPoints);
step = max(floor(n/2000),1);
IterMax    = 500
tol = gridSize/10;

r0 = 1/param.beta(1)
r = r0 +epsilon;

r = [r; r0.^2./r];
beta = 1./(r);



% Calculation: Case 1
param.beta = beta(:,1)
param1 = param;
[Vk1, kOpt1, Vy1, kOptY1]   = Viterate(param1, gridPoints, tol, IterMax);
maxK1  = max(max(kOpt1(:,2:end)));
 
1


% Calculation: Case 2
param.beta = beta(:,2)
param2 = param;
[Vk2, kOpt2, Vy2, kOptY2]   = Viterate(param2, gridPoints, tol, IterMax);
maxK2  = max(max(kOpt2(:,2:end)));

2

% Calculation: Case 3
param.beta = beta(:,3)
param3 = param;
[Vk3, kOpt3, Vy3, kOptY3]   = Viterate(param3, gridPoints, tol, IterMax);
maxK3  = max(max(kOpt3(:,2:end)));

3


% Calculation: Case 4
param.beta = beta(:,4)
param4 = param;
[Vk4, kOpt4, Vy4, kOptY4]   = Viterate(param4, gridPoints, tol, IterMax);
maxK4  = max(max(kOpt4(:,2:end)));

4



% Calculation: Case 5
param.beta = beta(:,5)
param5 = param;
[Vk5, kOpt5, Vy5, kOptY5]   = Viterate(param5, gridPoints, tol, IterMax);
maxK5  = max(max(kOpt5(:,2:end)));

5

% Calculation: State 6
% ---------------------

param.beta = beta(:,6)
param6 = param;
[Vk6, kOpt6, Vy6, kOptY6]   = Viterate(param6, gridPoints, tol, IterMax);
maxK6  = max(max(kOpt6(:,2:end)));

6

% Calculation: Case 7
param.beta = beta(:,7)
param7 = param;
[Vk7, kOpt7, Vy7, kOptY7]   = Viterate(param7, gridPoints, tol, IterMax);
maxK7  = max(max(kOpt7(:,2:end)));

7

save('BCBeta4.mat');

% Standard Plots
%%%%%%%%%%%%%%%%%

index = kOpt1(:,1)<= (1.3*max([maxK1; maxK2; maxK3]));
k0 = 1*[1 1 1 1];
state0 = [1 1 1 1];
T = 20;

N = sum(index);
reduction = max(1, floor(N/20000));
reducedIndex = 1:reduction:N;

yy = [f(kOpt1(reducedIndex,1), param1.s(1), param1),  f(kOpt1(reducedIndex,1), param1.s(2), param1)];



% Other Plots
%%%%%%%%%%%%%%%
BasePath = 'Graphs\';
folder   = '\CD4BetaBC\';
path     =  strcat(BasePath,folder);
mkdir(cd,path)


lw = 1;
lineTypes = '-|--|:|-.|*|.-|--|:';
set(0, 'DefaultAxesColorOrder',[0 0 1],...
      'DefaultAxesLineStyleOrder',lineTypes);
xLimits = [min(Vy1(index,1)) max(Vy1(index,1))];

% Value
VyS1 = [Vy1(:,1), Vy1(:,2), Vy2(:,2), Vy3(:,2), Vy4(:,2), Vy5(:,2), Vy6(:,2), Vy7(:,2)];
VyS1 = VyS1(index,:);
VyS2 = [Vy1(:,1), Vy1(:,3), Vy2(:,3), Vy3(:,3), Vy4(:,3), Vy5(:,3), Vy6(:,3), Vy7(:,3)];
VyS2 = VyS2(index,:);
yLimits = [-.01 round(max(max(VyS1))+0.5)]

figure; 
subplot(1,2,1);
plot(VyS1(:,1), VyS1(:,2:end), 'LineWidth', lw)
title('(a) Value Function: Boom');
ylabel('V');
xlabel('y');
xlim(xLimits);
ylim(yLimits);

subplot(1,2,2);
plot(VyS2(:,1), VyS2(:,2:end), 'LineWidth', lw)
title('(b) Value Function: Recession');
%ylabel('V');
xlabel('y');
xlim(xLimits);
ylim(yLimits);


% Save Graph
filename = 'Value4BC';

try
    saveas(gcf, strcat(path,filename) , 'epsc');
catch %if there is no directory create one
    mkdir(BasePath, folder)
    saveas(gcf, strcat(path,filename) , 'epsc');
end





% Capital / Dividends
KyS1 = [kOptY1(:,1), kOptY1(:,2), kOptY2(:,2), kOptY3(:,2), kOptY4(:,2), kOptY5(:,2), kOptY6(:,2), kOptY7(:,2)];
KyS1 = KyS1(index,:);
KyS2 = [kOptY1(:,1), kOptY1(:,3), kOptY2(:,3), kOptY3(:,3), kOptY4(:,3), kOptY5(:,3), kOptY6(:,3), kOptY7(:,3)];
KyS2 = KyS2(index,:);
yLimits = [-.01 xLimits(2)];

nS = size(kOptY1,2)-1;
Dy1  = [kOptY1(:,1), repmat(kOptY1(:,1),1,nS)- kOptY1(:,2:end)];
Dy2  = [kOptY2(:,1), repmat(kOptY2(:,1),1,nS)- kOptY2(:,2:end)];
Dy3  = [kOptY3(:,1), repmat(kOptY3(:,1),1,nS)- kOptY3(:,2:end)];
Dy4  = [kOptY4(:,1), repmat(kOptY4(:,1),1,nS)- kOptY4(:,2:end)];
Dy5  = [kOptY5(:,1), repmat(kOptY5(:,1),1,nS)- kOptY5(:,2:end)];
Dy6  = [kOptY6(:,1), repmat(kOptY6(:,1),1,nS)- kOptY6(:,2:end)];
Dy7  = [kOptY7(:,1), repmat(kOptY7(:,1),1,nS)- kOptY7(:,2:end)];


DyS1 = [Dy1(:,1), Dy1(:,2), Dy2(:,2), Dy3(:,2), Dy4(:,2), Dy5(:,2), Dy6(:,2), Dy7(:,2)];
DyS1 = DyS1(index,:);
DyS2 = [Dy1(:,1), Dy1(:,3), Dy2(:,3), Dy3(:,3), Dy4(:,3), Dy5(:,3), Dy6(:,3), Dy7(:,3)];
DyS2 = DyS2(index,:);

figure; 
subplot(2,1,1);
plot(KyS1(:,1), KyS1(:,2:end), 'LineWidth', lw)
title('(a) Optimal Capital in the Next Period in a Boom');
ylabel('k_{t+1}^*');
xlabel('y');
xlim(xLimits);
ylim(yLimits);

subplot(2,1,2);
plot(DyS1(:,1), DyS1(:,2:end), 'LineWidth', lw)
title('(b) Optimal Dividend Policy in a Boom');
ylabel('d^*');
xlabel('y');
xlim(xLimits);
ylim(yLimits);

filename ='CapitalDividend3S1'
saveas(gcf, strcat(path,filename) , 'epsc');

figure; 
subplot(2,1,1);
plot(KyS2(:,1), KyS2(:,2:end), 'LineWidth', lw)
title('(a) Optimal Capital in the Next Period in a Recession');
ylabel('k_{t+1}^*');
xlabel('y');
xlim(xLimits);
ylim(yLimits);

subplot(2,1,2);
plot(DyS2(:,1), DyS2(:,2:end), 'LineWidth', lw)
title('(b) Optimal Dividend Policy in a Recession');
ylabel('d^*');
xlabel('y');
xlim(xLimits);
ylim(yLimits);

filename ='CapitalDividend3S2'
saveas(gcf, strcat(path,filename) , 'epsc');


index = [1+1/gridSize, 1+3/gridSize, 1+6/gridSize];
Vgood = VyS1(index,:);
Vbad = VyS2(index,:);
Vpercent = (Vbad./Vgood-1)*100
Vaverage = (Vgood+Vbad)/2;

VZB = repmat(epsilon, 1, 3);
VZB = [VZB; 
        Vgood(1, 2:end), Vgood(2, 2:end), Vgood(3, 2:end);       
        Vbad(1, 2:end), Vbad(2, 2:end), Vbad(3, 2:end);
        Vaverage(1, 2:end), Vaverage(2, 2:end), Vaverage(3, 2:end);       
        Vpercent(1, 2:end), Vpercent(2, 2:end), Vpercent(3, 2:end)];
    
tempindex=[repmat(false, 1, size(DyS2, 2)); 
    (DyS2(1:end-1,:)-DyS2(2:end,:))>0.3];
temp = repmat(DyS2(:,1), 1, size(DyS2, 2));
temp = temp(tempindex)

kLD = epsilon;
kLD = [kLD; max(KyS1(:,2:end)); temp'];

cap = kLD(2,:);

for i=1:100
    cap = [cap; f(cap(end,:), param.s(2), param)];
end

kToDie = repmat(kLD(3,:), size(cap,1), 1);

kLD = [kLD;sum(cap > kToDie);sum(cap> 0)]


% Benchmark Capital
longKS1 = max(KyS1);
longKS1 = longKS1(2:end)

longKS2 = max(KyS2);
longKS2 = longKS2(2:end)

% Case 1
param = param1;
betaXp = [param.beta.*param.p(:,1), param.p*param.beta]';
kBench = 1./(betaXp) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))
kBench1 = kBench;

% Benchmark Value
temp =[kBench(2,1); kBench(2,1); kBench(1,2); kBench(1,2) ];
vd = f(temp, param.s([1; 2; 1; 2])',param)-param.s([1 1 2 2])';
vd(vd<0) = 0;
ytemp = vd;
vd = vd -temp([1 3 2 4]);

mbeta = diag([param.beta([1 2 1 2])]);

mP = [param.p([1 3]) 0 0; 0 0 param.p([2 4])];
mP = mP([1 2 1 2],:);

V1 = (diag([1 1 1 1])- mbeta * mP)^(-1) * vd-ytemp+1


% Case 2
param = param2;
betaXp = [param.beta.*param.p(:,1), param.p*param.beta]';
kBench = 1./(betaXp) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))
kBench2 = kBench;

% Benchmark Value
temp =[kBench(2,1); kBench(2,1); kBench(1,2); kBench(1,2) ];
vd = f(temp, param.s([1; 2; 1; 2])',param)-param.s([1 1 2 2])';
vd(vd<0) = 0;
ytemp = vd;
vd = vd -temp([1 3 2 4]);

mbeta = diag([param.beta([1 2 1 2])]);

mP = [param.p([1 3]) 0 0; 0 0 param.p([2 4])];
mP = mP([1 2 1 2],:);

V2 = (diag([1 1 1 1])- mbeta * mP)^(-1) * vd-ytemp+1


% Case 3
param = param3;
betaXp = [param.beta.*param.p(:,1), param.p*param.beta]';
kBench = 1./(betaXp) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))
kBench3 = kBench;

% Benchmark Value
temp =[kBench(2,1); kBench(2,1); kBench(1,2); kBench(1,2) ];
vd = f(temp, param.s([1; 2; 1; 2])',param)-param.s([1 1 2 2])';
vd(vd<0) = 0;
ytemp = vd;
vd = vd -temp([1 3 2 4]);

mbeta = diag([param.beta([1 2 1 2])]);

mP = [param.p([1 3]) 0 0; 0 0 param.p([2 4])];
mP = mP([1 2 1 2],:);

V3 = (diag([1 1 1 1])- mbeta * mP)^(-1) * vd-ytemp+1


% Case 4
param = param4;
betaXp = [param.beta.*param.p(:,1), param.p*param.beta]';
kBench = 1./(betaXp) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))
kBench4 = kBench;

% Benchmark Value
temp =[kBench(2,1); kBench(2,1); kBench(1,2); kBench(1,2) ];
vd = f(temp, param.s([1; 2; 1; 2])',param)-param.s([1 1 2 2])';
vd(vd<0) = 0;
ytemp = vd;
vd = vd -temp([1 3 2 4]);

mbeta = diag([param.beta([1 2 1 2])]);

mP = [param.p([1 3]) 0 0; 0 0 param.p([2 4])];
mP = mP([1 2 1 2],:);

V4 = (diag([1 1 1 1])- mbeta * mP)^(-1) * vd-ytemp+1
% Case 5
param = param5;
betaXp = [param.beta.*param.p(:,1), param.p*param.beta]';
kBench = 1./(betaXp) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))
kBench5 = kBench;

% Benchmark Value
temp =[kBench(2,1); kBench(2,1); kBench(1,2); kBench(1,2) ];
vd = f(temp, param.s([1; 2; 1; 2])',param)-param.s([1 1 2 2])';
vd(vd<0) = 0;
ytemp = vd;
vd = vd -temp([1 3 2 4]);

mbeta = diag([param.beta([1 2 1 2])]);

mP = [param.p([1 3]) 0 0; 0 0 param.p([2 4])];
mP = mP([1 2 1 2],:);

V5 = (diag([1 1 1 1])- mbeta * mP)^(-1) * vd-ytemp+1

% Case 6
param = param6;
betaXp = [param.beta.*param.p(:,1), param.p*param.beta]';
kBench = 1./(betaXp) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))
kBench6 = kBench;

% Benchmark Value
temp =[kBench(2,1); kBench(2,1); kBench(1,2); kBench(1,2) ];
vd = f(temp, param.s([1; 2; 1; 2])',param)-param.s([1 1 2 2])';
vd(vd<0) = 0;
ytemp = vd;
vd = vd -temp([1 3 2 4]);

mbeta = diag([param.beta([1 2 1 2])]);

mP = [param.p([1 3]) 0 0; 0 0 param.p([2 4])];
mP = mP([1 2 1 2],:);

V6 = (diag([1 1 1 1])- mbeta * mP)^(-1) * vd-ytemp+1


% Case 7
param = param7;
betaXp = [param.beta.*param.p(:,1), param.p*param.beta]';
kBench = 1./(betaXp) + param.delta;
kBench = (kBench./param.gamma./param.alpha).^(1./(param.alpha-1))
kBench7 = kBench;

% Benchmark Value
temp =[kBench(2,1); kBench(2,1); kBench(1,2); kBench(1,2) ];
vd = f(temp, param.s([1; 2; 1; 2])',param)-param.s([1 1 2 2])';
vd(vd<0) = 0;
ytemp = vd;
vd = vd -temp([1 3 2 4]);

mbeta = diag([param.beta([1 2 1 2])]);

mP = [param.p([1 3]) 0 0; 0 0 param.p([2 4])];
mP = mP([1 2 1 2],:);

V7 = (diag([1 1 1 1])- mbeta * mP)^(-1) * vd-ytemp+1

bb = [kBench1(2,1), kBench1(1,2), V1([1 4])';
    kBench2(2,1), kBench2(1,2), V2([1 4])';
    kBench3(2,1), kBench3(1,2), V3([1 4])';
    kBench4(2,1), kBench4(1,2), V4([1 4])';
    kBench5(2,1), kBench5(1,2), V5([1 4])';
    kBench6(2,1), kBench6(1,2), V6([1 4])';
    kBench7(2,1), kBench7(1,2), V7([1 4])']';

bb = [bb; (bb(3,:)+bb(4,:))/2; (bb(4,:)-bb(3,:))./bb(3,:)*100]

% Maximal constraint capital relative to benchmark
round( 10000*(kLD(2,:)./bb(1,:)-1))/100

save('BCBeta4.mat');

